from dtk.utils.Campaign.ClassValidator import ClassValidator
from dtk.utils.Campaign.utils.BaseCampaign import BaseCampaign
try:
    from CampaignEnum import *
except ImportError:
    from .CampaignEnum import *



class CampaignEvent(BaseCampaign):
    _definition = {'Event_Coordinator_Config': {'description': 'An object that specifies how the event is handled by the simulation. It specifies which Event Coordinator class will handle the event, and then configures the coordinator.', 'type': 'idmAbstractType:EventCoordinator'}, 'Nodeset_Config': {'description': 'An object that specifies in which nodes the interventions will be distributed. ', 'type': 'idmAbstractType:NodeSet'}, 'Start_Day': {'default': 1, 'description': "The day of the simulation to activate the event's event coordinator.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'class': 'CampaignEvent'}
    _validator = ClassValidator(_definition, 'CampaignEvent')

    def __init__(self, Event_Coordinator_Config=None, Nodeset_Config=None, Start_Day=1, **kwargs):
        super(CampaignEvent, self).__init__(**kwargs)
        self.Event_Coordinator_Config = Event_Coordinator_Config
        self.Nodeset_Config = Nodeset_Config
        self.Start_Day = Start_Day



class CampaignEventByYear(BaseCampaign):
    _definition = {'Event_Coordinator_Config': {'description': 'An object that specifies how the event is handled by the simulation. It specifies which Event Coordinator class will handle the event, and then configures the coordinator.', 'type': 'idmAbstractType:EventCoordinator'}, 'Nodeset_Config': {'description': 'An object that specifies in which nodes the interventions will be distributed. ', 'type': 'idmAbstractType:NodeSet'}, 'Start_Year': {'default': 1900, 'description': 'The year to start using the assortivity weighting matrix. ', 'max': 2200, 'min': 1900, 'type': 'float'}, 'class': 'CampaignEventByYear'}
    _validator = ClassValidator(_definition, 'CampaignEventByYear')

    def __init__(self, Event_Coordinator_Config=None, Nodeset_Config=None, Start_Year=1900, **kwargs):
        super(CampaignEventByYear, self).__init__(**kwargs)
        self.Event_Coordinator_Config = Event_Coordinator_Config
        self.Nodeset_Config = Nodeset_Config
        self.Start_Year = Start_Year



class CalendarEventCoordinator(BaseCampaign):
    _definition = {'Distribution_Coverages': {'ascending': 0, 'default': 0, 'description': 'A vector of floats for the fraction of individuals that will receive this intervention in a CalendarEventCoordinator.', 'max': 1, 'min': 0, 'type': 'Vector Float'}, 'Distribution_Times': {'default': 0, 'description': 'A vector of integers for simulation times at which distribution of events occurs in a CalendarEventCoordinator.', 'max': 2147480000.0, 'min': 1, 'type': 'Vector Int'}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'class': 'CalendarEventCoordinator'}
    _validator = ClassValidator(_definition, 'CalendarEventCoordinator')

    def __init__(self, Distribution_Coverages=0, Distribution_Times=0, Intervention_Config=None, Node_Property_Restrictions=[], Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=CalendarEventCoordinator_Target_Demographic_Enum.Everyone, Target_Gender=CalendarEventCoordinator_Target_Gender_Enum.All, Target_Residents_Only=False, **kwargs):
        super(CalendarEventCoordinator, self).__init__(**kwargs)
        self.Distribution_Coverages = Distribution_Coverages
        self.Distribution_Times = Distribution_Times
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only



class CommunityHealthWorkerEventCoordinator(BaseCampaign):
    _definition = {'Amount_In_Shipment': {'default': 2147480000.0, 'description': 'The number of interventions (such as vaccine doses) that a health worker or clinic receives in a shipment.', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'Days_Between_Shipments': {'default': 3.40282e+38, 'description': 'The number of days to wait before a clinic or health worker receives a new shipment of interventions (such as vaccine doses).', 'max': 3.40282e+38, 'min': 1, 'type': 'float'}, 'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Duration': {'default': 3.40282e+38, 'description': 'The number of days for an event coordinator to be active before it expires.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Initial_Amount': {'default': 6, 'depends-on': {'Initial_Amount_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'The initial amount of stock of interventions (such as vaccine doses). ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Initial_Amount_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The distribution type to set when initializing Initial_Amount.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Initial_Amount_Max': {'default': 1, 'depends-on': {'Initial_Amount_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum amount of initial stock when Initial_Amount_Distribution_Type is set to UNIFORM_DISTRIBUTION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Initial_Amount_Mean': {'default': 6, 'depends-on': {'Initial_Amount_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The average amount of initial stock when Initial_Amount_Distribution_Type is set to GAUSSIAN_DISTRIBUTION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Initial_Amount_Min': {'default': 0, 'depends-on': {'Initial_Amount_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum amount of initial stock when Initial_Amount_Distribution_Type is set to UNIFORM_DISTRIBUTION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Initial_Amount_Std_Dev': {'default': 1, 'depends-on': {'Initial_Amount_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation for the amount of initial stock when Initial_Amount_Distribution_Type is set to GAUSSIAN_DISTRIBUTION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Max_Distributed_Per_Day': {'default': 2147480000.0, 'description': 'The maximum number of interventions (such as vaccine doses) that can be distributed by health workers or clinics in a given day.', 'max': 2147480000.0, 'min': 1, 'type': 'integer'}, 'Max_Stock': {'default': 2147480000.0, 'description': 'The maximum number of interventions (such as vaccine doses) that can be stored by a health worker or clinic.', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Trigger_Condition_List': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': "The list of events that are of interest to the community health worker (CHW). If one of these events occurs, the individual or node is put into a queue to receive the CHW's intervention. The CHW processes the queue when the event coordinator is updated.", 'type': 'Vector String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Waiting_Period': {'default': 3.40282e+38, 'description': 'The number of days a person or node can be in the queue waiting to get the intervention from the community health worker (CHW).', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'class': 'CommunityHealthWorkerEventCoordinator'}
    _validator = ClassValidator(_definition, 'CommunityHealthWorkerEventCoordinator')

    def __init__(self, Amount_In_Shipment=2147480000.0, Days_Between_Shipments=3.40282e+38, Demographic_Coverage=1, Duration=3.40282e+38, Initial_Amount=6, Initial_Amount_Distribution_Type=CommunityHealthWorkerEventCoordinator_Initial_Amount_Distribution_Type_Enum.NOT_INITIALIZED, Initial_Amount_Max=1, Initial_Amount_Mean=6, Initial_Amount_Min=0, Initial_Amount_Std_Dev=1, Intervention_Config=None, Max_Distributed_Per_Day=2147480000.0, Max_Stock=2147480000.0, Node_Property_Restrictions=[], Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=CommunityHealthWorkerEventCoordinator_Target_Demographic_Enum.Everyone, Target_Gender=CommunityHealthWorkerEventCoordinator_Target_Gender_Enum.All, Target_Residents_Only=False, Trigger_Condition_List='NoTrigger', Waiting_Period=3.40282e+38, **kwargs):
        super(CommunityHealthWorkerEventCoordinator, self).__init__(**kwargs)
        self.Amount_In_Shipment = Amount_In_Shipment
        self.Days_Between_Shipments = Days_Between_Shipments
        self.Demographic_Coverage = Demographic_Coverage
        self.Duration = Duration
        self.Initial_Amount = Initial_Amount
        self.Initial_Amount_Distribution_Type = Initial_Amount_Distribution_Type.name if isinstance(Initial_Amount_Distribution_Type, Enum) else Initial_Amount_Distribution_Type
        self.Initial_Amount_Max = Initial_Amount_Max
        self.Initial_Amount_Mean = Initial_Amount_Mean
        self.Initial_Amount_Min = Initial_Amount_Min
        self.Initial_Amount_Std_Dev = Initial_Amount_Std_Dev
        self.Intervention_Config = Intervention_Config
        self.Max_Distributed_Per_Day = Max_Distributed_Per_Day
        self.Max_Stock = Max_Stock
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Trigger_Condition_List = Trigger_Condition_List
        self.Waiting_Period = Waiting_Period



class CoverageByNodeEventCoordinator(BaseCampaign):
    _definition = {'Coverage_By_Node': {'description': 'An array of (nodeID, coverage) pairs configuring the demographic coverage of interventions by node for the targeted populations. The coverage value must be a float between 0 and 1.', 'type': 'list', 'item_type': {'type': 'object'}, 'default': []}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Number_Repetitions': {'default': 1, 'description': 'The number of times an intervention is given, used with Timesteps_Between_Repetitions.', 'max': 1000, 'min': -1, 'type': 'integer'}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Timesteps_Between_Repetitions': {'default': -1, 'description': 'The repetition interval.', 'max': 10000, 'min': -1, 'type': 'integer'}, 'class': 'CoverageByNodeEventCoordinator'}
    _validator = ClassValidator(_definition, 'CoverageByNodeEventCoordinator')

    def __init__(self, Coverage_By_Node=[], Intervention_Config=None, Node_Property_Restrictions=[], Number_Repetitions=1, Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=CoverageByNodeEventCoordinator_Target_Demographic_Enum.Everyone, Target_Gender=CoverageByNodeEventCoordinator_Target_Gender_Enum.All, Target_Residents_Only=False, Timesteps_Between_Repetitions=-1, **kwargs):
        super(CoverageByNodeEventCoordinator, self).__init__(**kwargs)
        self.Coverage_By_Node = Coverage_By_Node
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Number_Repetitions = Number_Repetitions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Timesteps_Between_Repetitions = Timesteps_Between_Repetitions



class GroupInterventionDistributionEventCoordinator(BaseCampaign):
    _definition = {'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Number_Repetitions': {'default': 1, 'description': 'The number of times an intervention is given, used with Timesteps_Between_Repetitions.', 'max': 1000, 'min': -1, 'type': 'integer'}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Disease_State': {'default': 'Everyone', 'description': 'The disease state group targeted by this intervention.', 'enum': ['Everyone', 'Infected', 'ActiveInfection', 'LatentInfection', 'MDR', 'TreatmentNaive', 'HasFailedTreatment', 'HIVNegative', 'ActiveHadTreatment'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Timesteps_Between_Repetitions': {'default': -1, 'description': 'The repetition interval.', 'max': 10000, 'min': -1, 'type': 'integer'}, 'class': 'GroupInterventionDistributionEventCoordinator'}
    _validator = ClassValidator(_definition, 'GroupInterventionDistributionEventCoordinator')

    def __init__(self, Demographic_Coverage=1, Intervention_Config=None, Node_Property_Restrictions=[], Number_Repetitions=1, Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=GroupInterventionDistributionEventCoordinator_Target_Demographic_Enum.Everyone, Target_Disease_State=GroupInterventionDistributionEventCoordinator_Target_Disease_State_Enum.Everyone, Target_Gender=GroupInterventionDistributionEventCoordinator_Target_Gender_Enum.All, Target_Residents_Only=False, Timesteps_Between_Repetitions=-1, **kwargs):
        super(GroupInterventionDistributionEventCoordinator, self).__init__(**kwargs)
        self.Demographic_Coverage = Demographic_Coverage
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Number_Repetitions = Number_Repetitions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Disease_State = Target_Disease_State.name if isinstance(Target_Disease_State, Enum) else Target_Disease_State
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Timesteps_Between_Repetitions = Timesteps_Between_Repetitions



class GroupInterventionDistributionEventCoordinatorHIV(BaseCampaign):
    _definition = {'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Number_Repetitions': {'default': 1, 'description': 'The number of times an intervention is given, used with Timesteps_Between_Repetitions.', 'max': 1000, 'min': -1, 'type': 'integer'}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Time_Offset': {'default': 0, 'description': 'This offset is used to determine the demographic coverage for HIVCoinfectionDistribution (used with GroupInterventionDistributionEventCoordinatorHIV), which is defined in the demographics file.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Timesteps_Between_Repetitions': {'default': -1, 'description': 'The repetition interval.', 'max': 10000, 'min': -1, 'type': 'integer'}, 'class': 'GroupInterventionDistributionEventCoordinatorHIV'}
    _validator = ClassValidator(_definition, 'GroupInterventionDistributionEventCoordinatorHIV')

    def __init__(self, Intervention_Config=None, Node_Property_Restrictions=[], Number_Repetitions=1, Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=GroupInterventionDistributionEventCoordinatorHIV_Target_Demographic_Enum.Everyone, Target_Gender=GroupInterventionDistributionEventCoordinatorHIV_Target_Gender_Enum.All, Target_Residents_Only=False, Time_Offset=0, Timesteps_Between_Repetitions=-1, **kwargs):
        super(GroupInterventionDistributionEventCoordinatorHIV, self).__init__(**kwargs)
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Number_Repetitions = Number_Repetitions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Time_Offset = Time_Offset
        self.Timesteps_Between_Repetitions = Timesteps_Between_Repetitions



class IncidenceEventCoordinator(BaseCampaign):
    _definition = {'Incidence_Counter': {'description': 'List of JSON objects for specifying the conditions and parameters that must be met for an incidence to be counted.', 'Count_Events_For_Num_Timesteps': {'default': 1, 'description': 'If set to true (1), then the waning effect values, as specified in the Effect_List list of objects for the WaningEffectCombo classes, are added together. If set to false (0), the waning effect values are multiplied. The resulting waning effect value cannot be greater than 1.', 'max': 2147480000.0, 'min': 1, 'type': 'integer'}, 'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Trigger_Condition_List': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'A list of events that will trigger an intervention.', 'type': 'Vector String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}}, 'Number_Repetitions': {'default': 1, 'description': 'The number of times an intervention is given, used with Timesteps_Between_Repetitions.', 'max': 1000, 'min': -1, 'type': 'integer'}, 'Responder': {'description': 'List of JSON objects for specifying the the threshold type and values and the actions to take when the specified conditions and parameters have been met, as configured in the Incidence_Counter parameter.', 'Action_List': {'description': 'List (array) of JSON objects, including the values specified in the following parameters: * Threshold, * Event_To_Broadcast.', '<Action Value>': {'Event_To_Broadcast': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The action event to occur when the specified trigger value in the Threshold parameter is met. At least one action must be defined for Event_To_Broadcast.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Threshold': {'default': 0, 'description': 'The threshold value that triggers the specified action for the Event_To_Broadcast parameter.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}}}, 'Threshold_Type': {'default': 'COUNT', 'description': 'Threshold type. Possible values are: * COUNT, * PERCENTAGE.', 'enum': ['COUNT', 'PERCENTAGE'], 'type': 'enum'}}, 'Timesteps_Between_Repetitions': {'default': -1, 'description': 'The repetition interval.', 'max': 10000, 'min': -1, 'type': 'integer'}, 'class': 'IncidenceEventCoordinator'}
    _validator = ClassValidator(_definition, 'IncidenceEventCoordinator')

    def __init__(self, Incidence_Counter=None, Number_Repetitions=1, Responder=None, Timesteps_Between_Repetitions=-1, **kwargs):
        super(IncidenceEventCoordinator, self).__init__(**kwargs)
        self.Incidence_Counter = Incidence_Counter
        self.Number_Repetitions = Number_Repetitions
        self.Responder = Responder
        self.Timesteps_Between_Repetitions = Timesteps_Between_Repetitions



class NChooserEventCoordinator(BaseCampaign):
    _definition = {'Distributions': {'description': 'The ordered list of elements defining when, to whom, and how many interventions to distribute.', '<TargetedDistributionSTI Value>': {'Age_Ranges_Years': {'description': 'A list of age ranges that individuals must be in to qualify for an intervention. Each age range is a JSON object with a minimum and a maximum property. ', '<AgeRange Value>': {'Max': {'default': 125, 'description': 'This parameter determines the maximum age, in years for individuals to be included in the targeted population. An individual is considered in range if their age is greater than or equal to the minimum age and less than the maximum age, in floating point years of age.', 'max': 125, 'min': 0, 'type': 'float'}, 'Min': {'default': 0, 'description': 'This parameter determines the minimum age, in years for individuals to be included in the targeted population. An individual is considered in range if their age is greater than or equal to the minimum age and less than the maximum age, in floating point years of age.', 'max': 125, 'min': 0, 'type': 'float'}}}, 'End_Year': {'default': 2200, 'description': 'The year to stop distributing the intervention.', 'max': 2200, 'min': 1900, 'type': 'float'}, 'Num_Targeted': {'default': 0, 'description': 'The number of individuals to target with the intervention.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Num_Targeted_Females': {'default': 0, 'description': 'The number of female individuals to distribute interventions to during this time period.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Num_Targeted_Males': {'default': 0, 'description': 'The number of male individuals to distribute interventions to during this time period.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Start_Year': {'default': 1900, 'description': 'The year to start distributing the intervention.', 'max': 2200, 'min': 1900, 'type': 'float'}}}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'class': 'NChooserEventCoordinator'}
    _validator = ClassValidator(_definition, 'NChooserEventCoordinator')

    def __init__(self, Distributions=None, Intervention_Config=None, **kwargs):
        super(NChooserEventCoordinator, self).__init__(**kwargs)
        self.Distributions = Distributions
        self.Intervention_Config = Intervention_Config



class NChooserEventCoordinatorHIV(BaseCampaign):
    _definition = {'Distributions': {'description': 'The ordered list of elements defining when, to whom, and how many interventions to distribute.', '<TargetedDistributionSTI Value>': {'Age_Ranges_Years': {'description': 'A list of age ranges that individuals must be in to qualify for an intervention. Each age range is a JSON object with a minimum and a maximum property. ', '<AgeRange Value>': {'Max': {'default': 125, 'description': 'This parameter determines the maximum age, in years for individuals to be included in the targeted population. An individual is considered in range if their age is greater than or equal to the minimum age and less than the maximum age, in floating point years of age.', 'max': 125, 'min': 0, 'type': 'float'}, 'Min': {'default': 0, 'description': 'This parameter determines the minimum age, in years for individuals to be included in the targeted population. An individual is considered in range if their age is greater than or equal to the minimum age and less than the maximum age, in floating point years of age.', 'max': 125, 'min': 0, 'type': 'float'}}}, 'End_Year': {'default': 2200, 'description': 'The year to stop distributing the intervention.', 'max': 2200, 'min': 1900, 'type': 'float'}, 'Num_Targeted': {'default': 0, 'description': 'The number of individuals to target with the intervention.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Num_Targeted_Females': {'default': 0, 'description': 'The number of female individuals to distribute interventions to during this time period.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Num_Targeted_Males': {'default': 0, 'description': 'The number of male individuals to distribute interventions to during this time period.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Start_Year': {'default': 1900, 'description': 'The year to start distributing the intervention.', 'max': 2200, 'min': 1900, 'type': 'float'}}}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'class': 'NChooserEventCoordinatorHIV'}
    _validator = ClassValidator(_definition, 'NChooserEventCoordinatorHIV')

    def __init__(self, Distributions=None, Intervention_Config=None, **kwargs):
        super(NChooserEventCoordinatorHIV, self).__init__(**kwargs)
        self.Distributions = Distributions
        self.Intervention_Config = Intervention_Config



class NChooserEventCoordinatorSTI(BaseCampaign):
    _definition = {'Distributions': {'description': 'The ordered list of elements defining when, to whom, and how many interventions to distribute.', '<TargetedDistributionSTI Value>': {'Age_Ranges_Years': {'description': 'A list of age ranges that individuals must be in to qualify for an intervention. Each age range is a JSON object with a minimum and a maximum property. ', '<AgeRange Value>': {'Max': {'default': 125, 'description': 'This parameter determines the maximum age, in years for individuals to be included in the targeted population. An individual is considered in range if their age is greater than or equal to the minimum age and less than the maximum age, in floating point years of age.', 'max': 125, 'min': 0, 'type': 'float'}, 'Min': {'default': 0, 'description': 'This parameter determines the minimum age, in years for individuals to be included in the targeted population. An individual is considered in range if their age is greater than or equal to the minimum age and less than the maximum age, in floating point years of age.', 'max': 125, 'min': 0, 'type': 'float'}}}, 'End_Year': {'default': 2200, 'description': 'The year to stop distributing the intervention.', 'max': 2200, 'min': 1900, 'type': 'float'}, 'Num_Targeted': {'default': 0, 'description': 'The number of individuals to target with the intervention.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Num_Targeted_Females': {'default': 0, 'description': 'The number of female individuals to distribute interventions to during this time period.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Num_Targeted_Males': {'default': 0, 'description': 'The number of male individuals to distribute interventions to during this time period.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Start_Year': {'default': 1900, 'description': 'The year to start distributing the intervention.', 'max': 2200, 'min': 1900, 'type': 'float'}}}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'class': 'NChooserEventCoordinatorSTI'}
    _validator = ClassValidator(_definition, 'NChooserEventCoordinatorSTI')

    def __init__(self, Distributions=None, Intervention_Config=None, **kwargs):
        super(NChooserEventCoordinatorSTI, self).__init__(**kwargs)
        self.Distributions = Distributions
        self.Intervention_Config = Intervention_Config



class ReferenceTrackingEventCoordinator(BaseCampaign):
    _definition = {'End_Year': {'default': 2200, 'description': 'The final date at which this set of targeted coverages should be applied (expiration).', 'max': 2200, 'min': 1900, 'type': 'float'}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Time_Value_Map': {'description': 'Map of times (years) to coverages.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Update_Period': {'default': 365, 'description': 'The time between distribution updates.', 'max': 3650, 'min': 1, 'type': 'float'}, 'class': 'ReferenceTrackingEventCoordinator'}
    _validator = ClassValidator(_definition, 'ReferenceTrackingEventCoordinator')

    def __init__(self, End_Year=2200, Intervention_Config=None, Node_Property_Restrictions=[], Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=ReferenceTrackingEventCoordinator_Target_Demographic_Enum.Everyone, Target_Gender=ReferenceTrackingEventCoordinator_Target_Gender_Enum.All, Target_Residents_Only=False, Time_Value_Map=None, Update_Period=365, **kwargs):
        super(ReferenceTrackingEventCoordinator, self).__init__(**kwargs)
        self.End_Year = End_Year
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Time_Value_Map = Time_Value_Map
        self.Update_Period = Update_Period



class ReferenceTrackingEventCoordinatorHIV(BaseCampaign):
    _definition = {'End_Year': {'default': 2200, 'description': 'The final date at which this set of targeted coverages should be applied (expiration).', 'max': 2200, 'min': 1900, 'type': 'float'}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Disease_State': {'default': 'Everyone', 'description': 'An array of particular disease states used in the ReferenceTrackingEventCoordinatorHIV. ', 'enum': ['Everyone', 'HIV_Positive', 'HIV_Negative', 'Tested_Positive', 'Tested_Negative', 'Not_Tested_Or_Tested_Negative'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Time_Value_Map': {'description': 'Map of times (years) to coverages.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Update_Period': {'default': 365, 'description': 'The time between distribution updates.', 'max': 3650, 'min': 1, 'type': 'float'}, 'class': 'ReferenceTrackingEventCoordinatorHIV'}
    _validator = ClassValidator(_definition, 'ReferenceTrackingEventCoordinatorHIV')

    def __init__(self, End_Year=2200, Intervention_Config=None, Node_Property_Restrictions=[], Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=ReferenceTrackingEventCoordinatorHIV_Target_Demographic_Enum.Everyone, Target_Disease_State=ReferenceTrackingEventCoordinatorHIV_Target_Disease_State_Enum.Everyone, Target_Gender=ReferenceTrackingEventCoordinatorHIV_Target_Gender_Enum.All, Target_Residents_Only=False, Time_Value_Map=None, Update_Period=365, **kwargs):
        super(ReferenceTrackingEventCoordinatorHIV, self).__init__(**kwargs)
        self.End_Year = End_Year
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Disease_State = Target_Disease_State.name if isinstance(Target_Disease_State, Enum) else Target_Disease_State
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Time_Value_Map = Time_Value_Map
        self.Update_Period = Update_Period



class StandardInterventionDistributionEventCoordinator(BaseCampaign):
    _definition = {'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Config': {'description': 'The nested JSON of the actual intervention to be distributed by this event coordinator.', 'type': 'idmAbstractType:Intervention'}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Number_Repetitions': {'default': 1, 'description': 'The number of times an intervention is given, used with Timesteps_Between_Repetitions.', 'max': 1000, 'min': -1, 'type': 'integer'}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Timesteps_Between_Repetitions': {'default': -1, 'description': 'The repetition interval.', 'max': 10000, 'min': -1, 'type': 'integer'}, 'class': 'StandardInterventionDistributionEventCoordinator'}
    _validator = ClassValidator(_definition, 'StandardInterventionDistributionEventCoordinator')

    def __init__(self, Demographic_Coverage=1, Intervention_Config=None, Node_Property_Restrictions=[], Number_Repetitions=1, Property_Restrictions=[], Property_Restrictions_Within_Node=[], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=StandardInterventionDistributionEventCoordinator_Target_Demographic_Enum.Everyone, Target_Gender=StandardInterventionDistributionEventCoordinator_Target_Gender_Enum.All, Target_Residents_Only=False, Timesteps_Between_Repetitions=-1, **kwargs):
        super(StandardInterventionDistributionEventCoordinator, self).__init__(**kwargs)
        self.Demographic_Coverage = Demographic_Coverage
        self.Intervention_Config = Intervention_Config
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Number_Repetitions = Number_Repetitions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Timesteps_Between_Repetitions = Timesteps_Between_Repetitions



class ARTBasic(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 1, 'description': 'Unit cost per drug (unamortized).', 'max': 99999, 'min': 0, 'type': 'float'}, 'Days_To_Achieve_Viral_Suppression': {'default': 183, 'description': 'The number of days after ART initiation over which infectiousness declines linearly until the ART_Viral_Suppression_Multiplier takes full effect.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'ARTBasic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM', 'TBHIV_SIM'], 'Viral_Suppression': {'default': 1, 'description': 'If set to true (1), ART will suppress viral load and extend prognosis.', 'type': 'bool'}, 'class': 'ARTBasic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ARTBasic')

    def __init__(self, Cost_To_Consumer=1, Days_To_Achieve_Viral_Suppression=183, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='ARTBasic', New_Property_Value=None, Sim_Types=['HIV_SIM', 'TBHIV_SIM'], Viral_Suppression=True, iv_type='IndividualTargeted', **kwargs):
        super(ARTBasic, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Achieve_Viral_Suppression = Days_To_Achieve_Viral_Suppression
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Viral_Suppression = Viral_Suppression
        self.iv_type = iv_type



class ARTDropout(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 1, 'description': 'Unit cost per drug (unamortized).', 'max': 99999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'ARTDropout', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM', 'TBHIV_SIM'], 'class': 'ARTDropout', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ARTDropout')

    def __init__(self, Cost_To_Consumer=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='ARTDropout', New_Property_Value=None, Sim_Types=['HIV_SIM', 'TBHIV_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(ARTDropout, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class ActiveDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'ActiveDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['TBHIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'ActiveDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ActiveDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=ActiveDiagnostic_Event_Or_Config_Enum.Config, Intervention_Name='ActiveDiagnostic', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['TBHIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(ActiveDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class AdherentDrug(BaseCampaign):
    _definition = {'Adherence_Config': {'description': 'A list of nested JSON objects for the interventions to be distributed by this event coordinator.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per drug (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Dosing_Type': {'default': 'SingleDose', 'description': 'The type of anti-malarial dosing to distribute in a drug intervention.', 'enum': ['SingleDose', 'FullTreatmentCourse', 'Prophylaxis', 'SingleDoseWhenSymptom', 'FullTreatmentWhenSymptom', 'SingleDoseParasiteDetect', 'FullTreatmentParasiteDetect', 'SingleDoseNewDetectionTech', 'FullTreatmentNewDetectionTech'], 'type': 'enum'}, 'Drug_Type': {'default': 'UNINITIALIZED STRING', 'description': 'The type of drug to distribute in a drugs intervention.', 'type': 'Constrained String', 'value_source': '<configuration>:Malaria_Drug_Params.*'}, 'Intervention_Name': {'default': 'AdherentDrug', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Max_Dose_Consideration_Duration': {'default': 3.40282e+38, 'description': 'The maximum number of days that an individual will consider taking the doses of the drug.', 'max': 3.40282e+38, 'min': 0.0416667, 'type': 'float'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Non_Adherence_Distribution': {'ascending': 0, 'default': 0, 'description': 'The non adherence probability value(s) assigned to the corresponding options in Non_Adherence_Options. The sum of non adherence distribution values must equal a total of 1.', 'max': 1, 'min': 0, 'type': 'Vector Float'}, 'Non_Adherence_Options': {'default': 'NEXT_UPDATE', 'description': 'Defines the action the person takes if they do not take a particular dose, are not adherent.', 'enum': ['NEXT_UPDATE', 'NEXT_DOSAGE_TIME', 'LOST_TAKE_NEXT', 'STOP'], 'type': 'Vector Enum'}, 'Sim_Types': ['MALARIA_SIM'], 'Took_Dose_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event that triggers the drug intervention campaign.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'class': 'AdherentDrug', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'AdherentDrug')

    def __init__(self, Adherence_Config=None, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Dosing_Type=AdherentDrug_Dosing_Type_Enum.SingleDose, Drug_Type='UNINITIALIZED STRING', Intervention_Name='AdherentDrug', Max_Dose_Consideration_Duration=3.40282e+38, New_Property_Value=None, Non_Adherence_Distribution=0, Non_Adherence_Options='NEXT_UPDATE', Sim_Types=['MALARIA_SIM'], Took_Dose_Event='NoTrigger', iv_type='IndividualTargeted', **kwargs):
        super(AdherentDrug, self).__init__(**kwargs)
        self.Adherence_Config = Adherence_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Dosing_Type = Dosing_Type.name if isinstance(Dosing_Type, Enum) else Dosing_Type
        self.Drug_Type = Drug_Type
        self.Intervention_Name = Intervention_Name
        self.Max_Dose_Consideration_Duration = Max_Dose_Consideration_Duration
        self.New_Property_Value = New_Property_Value
        self.Non_Adherence_Distribution = Non_Adherence_Distribution
        self.Non_Adherence_Options = Non_Adherence_Options
        self.Sim_Types = Sim_Types
        self.Took_Dose_Event = Took_Dose_Event
        self.iv_type = iv_type



class AgeDiagnostic(BaseCampaign):
    _definition = {'Age_Thresholds': {'description': 'Used to associate age ranges for individuals.', 'type': 'list', 'item_type': {'type': 'object'}, 'default': []}, 'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'AgeDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'AgeDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'AgeDiagnostic')

    def __init__(self, Age_Thresholds=[], Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='AgeDiagnostic', New_Property_Value=None, Sim_Types=['HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(AgeDiagnostic, self).__init__(**kwargs)
        self.Age_Thresholds = Age_Thresholds
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class AntiTBDrug(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 1, 'description': 'Unit cost per drug (unamortized).', 'max': 99999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Dose_Interval': {'default': 1, 'description': 'The interval between doses, in days.', 'max': 99999, 'min': 0, 'type': 'float'}, 'Drug_CMax': {'default': 1, 'description': 'The maximum drug concentration that can be used, and is in the same units as Drug_PKPD_C50.', 'max': 10000, 'min': 0, 'type': 'float'}, 'Drug_PKPD_C50': {'default': 1, 'description': 'The concentration at which drug killing rates are half of the maximum. Must use the same units as Drug_Cmax.', 'max': 5000, 'min': 0, 'type': 'float'}, 'Drug_Type': {'default': 'DOTS', 'description': 'Specifies the name of the drug treatment, such as DOTS or EmpiricTreatment, which can be useful for reporting from the simulation.', 'enum': ['DOTS', 'DOTSImproved', 'EmpiricTreatment', 'FirstLineCombo', 'SecondLineCombo', 'ThirdLineCombo', 'LatentTreatment'], 'type': 'enum'}, 'Drug_Vd': {'default': 1, 'description': 'The volume of drug distribution. This value is the ratio of the volume of the second compartment to the volume of the first compartment in a two-compartment model, and is dimensionless.', 'max': 10000, 'min': 0, 'type': 'float'}, 'Durability_Profile': {'default': 'FIXED_DURATION_CONSTANT_EFFECT', 'description': 'The profile of durability decay.', 'enum': ['FIXED_DURATION_CONSTANT_EFFECT', 'CONCENTRATION_VERSUS_TIME'], 'type': 'enum'}, 'Fraction_Defaulters': {'default': 0, 'description': 'The fraction of individuals who will not finish their drug treatment.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'AntiTBDrug', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Primary_Decay_Time_Constant': {'default': 1, 'description': 'The primary decay time constant (in days) of the decay profile.', 'max': 999999, 'min': 0, 'type': 'float'}, 'Reduced_Transmit': {'default': 1, 'description': 'The transmission reduction ratio, or the reduced infectiousness when an individual receives drugs.', 'max': 1, 'min': 0, 'type': 'float'}, 'Remaining_Doses': {'default': 0, 'description': 'The remaining doses in an intervention; enter a negative number for unlimited doses.', 'max': 999999, 'min': -1, 'type': 'integer'}, 'Secondary_Decay_Time_Constant': {'default': 1, 'description': 'The secondary decay time constant of the durability profile. ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Sim_Types': ['TBHIV_SIM'], 'TB_Drug_Cure_Rate': {'default': 0, 'description': 'The daily probability of TB cure in an individual with drug-sensitive TB under drug treatment.', 'max': 1, 'min': 0, 'type': 'float'}, 'TB_Drug_Inactivation_Rate': {'default': 0, 'description': 'The daily rate at which treatment with an anti-TB drug causes inactivation in an individual with drug-sensitive TB.', 'max': 1, 'min': 0, 'type': 'float'}, 'TB_Drug_Mortality_Rate': {'default': 0, 'description': 'The daily rate at which treatment with an anti-TB drug causes death  in an individual with drug-sensitive TB.', 'max': 1, 'min': 0, 'type': 'float'}, 'TB_Drug_Relapse_Rate': {'default': 0, 'description': 'The daily probability of TB inactivation and subsequent relapse in an individual with drug-sensitive TB under drug treatment.', 'max': 1, 'min': 0, 'type': 'float'}, 'TB_Drug_Resistance_Rate': {'default': 0, 'description': 'The daily probability that an individual with drug-sensitive TB will acquire MDR-TB under drug treatment. Only individuals who return to the latent state or fail can acquire MDR-TB.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'AntiTBDrug', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'AntiTBDrug')

    def __init__(self, Cost_To_Consumer=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Dose_Interval=1, Drug_CMax=1, Drug_PKPD_C50=1, Drug_Type=AntiTBDrug_Drug_Type_Enum.DOTS, Drug_Vd=1, Durability_Profile=AntiTBDrug_Durability_Profile_Enum.FIXED_DURATION_CONSTANT_EFFECT, Fraction_Defaulters=0, Intervention_Name='AntiTBDrug', New_Property_Value=None, Primary_Decay_Time_Constant=1, Reduced_Transmit=1, Remaining_Doses=0, Secondary_Decay_Time_Constant=1, Sim_Types=['TBHIV_SIM'], TB_Drug_Cure_Rate=0, TB_Drug_Inactivation_Rate=0, TB_Drug_Mortality_Rate=0, TB_Drug_Relapse_Rate=0, TB_Drug_Resistance_Rate=0, iv_type='IndividualTargeted', **kwargs):
        super(AntiTBDrug, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Dose_Interval = Dose_Interval
        self.Drug_CMax = Drug_CMax
        self.Drug_PKPD_C50 = Drug_PKPD_C50
        self.Drug_Type = Drug_Type.name if isinstance(Drug_Type, Enum) else Drug_Type
        self.Drug_Vd = Drug_Vd
        self.Durability_Profile = Durability_Profile.name if isinstance(Durability_Profile, Enum) else Durability_Profile
        self.Fraction_Defaulters = Fraction_Defaulters
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Primary_Decay_Time_Constant = Primary_Decay_Time_Constant
        self.Reduced_Transmit = Reduced_Transmit
        self.Remaining_Doses = Remaining_Doses
        self.Secondary_Decay_Time_Constant = Secondary_Decay_Time_Constant
        self.Sim_Types = Sim_Types
        self.TB_Drug_Cure_Rate = TB_Drug_Cure_Rate
        self.TB_Drug_Inactivation_Rate = TB_Drug_Inactivation_Rate
        self.TB_Drug_Mortality_Rate = TB_Drug_Mortality_Rate
        self.TB_Drug_Relapse_Rate = TB_Drug_Relapse_Rate
        self.TB_Drug_Resistance_Rate = TB_Drug_Resistance_Rate
        self.iv_type = iv_type



class AntimalarialDrug(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per drug (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Dosing_Type': {'default': 'SingleDose', 'description': 'The type of anti-malarial dosing to distribute in a drug intervention.', 'enum': ['SingleDose', 'FullTreatmentCourse', 'Prophylaxis', 'SingleDoseWhenSymptom', 'FullTreatmentWhenSymptom', 'SingleDoseParasiteDetect', 'FullTreatmentParasiteDetect', 'SingleDoseNewDetectionTech', 'FullTreatmentNewDetectionTech'], 'type': 'enum'}, 'Drug_Type': {'default': 'UNINITIALIZED STRING', 'description': 'The type of drug to distribute in a drugs intervention.', 'type': 'Constrained String', 'value_source': '<configuration>:Malaria_Drug_Params.*'}, 'Intervention_Name': {'default': 'AntimalarialDrug', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['MALARIA_SIM'], 'class': 'AntimalarialDrug', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'AntimalarialDrug')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Dosing_Type=AntimalarialDrug_Dosing_Type_Enum.SingleDose, Drug_Type='UNINITIALIZED STRING', Intervention_Name='AntimalarialDrug', New_Property_Value=None, Sim_Types=['MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(AntimalarialDrug, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Dosing_Type = Dosing_Type.name if isinstance(Dosing_Type, Enum) else Dosing_Type
        self.Drug_Type = Drug_Type
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class ArtificialDietHousingModification(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of pre-feed mosquito repellency and waning for housing modification.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per housing modification (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'ArtificialDietHousingModification', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for housing modification. Killing is conditional on NOT blocking the mosquito before feeding.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'ArtificialDietHousingModification', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ArtificialDietHousingModification')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='ArtificialDietHousingModification', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(ArtificialDietHousingModification, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class BCGVaccine(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Efficacy_Is_Multiplicative': {'default': 1, 'description': 'The overall vaccine efficacy when individuals receive more than one vaccine. When set to true (1), the vaccine efficacies are multiplied together; when set to false (0), the efficacies are additive.', 'type': 'bool'}, 'Intervention_Name': {'default': 'BCGVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['TBHIV_SIM'], 'Vaccine_Take': {'default': 1, 'description': 'The rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.', 'max': 1, 'min': 0, 'type': 'float'}, 'Vaccine_Take_Age_Decay_Rate': {'default': 1, 'description': 'The exponential decline in vaccine take over time. The rate of decline is in units of 1/years.', 'max': 3.40282e+38, 'min': -3.40282e+38, 'type': 'float'}, 'Vaccine_Type': {'default': 'Generic', 'description': 'The type of vaccine to distribute in a vaccine intervention.', 'enum': ['Generic', 'TransmissionBlocking', 'AcquisitionBlocking', 'MortalityBlocking'], 'type': 'enum'}, 'Waning_Config': {'description': 'The configuration of Ivermectin killing efficacy and waning over time.', 'type': 'idmType:WaningEffect'}, 'class': 'BCGVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'BCGVaccine')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Efficacy_Is_Multiplicative=True, Intervention_Name='BCGVaccine', New_Property_Value=None, Sim_Types=['TBHIV_SIM'], Vaccine_Take=1, Vaccine_Take_Age_Decay_Rate=1, Vaccine_Type=BCGVaccine_Vaccine_Type_Enum.Generic, Waning_Config=[], iv_type='IndividualTargeted', **kwargs):
        super(BCGVaccine, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Efficacy_Is_Multiplicative = Efficacy_Is_Multiplicative
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Vaccine_Take = Vaccine_Take
        self.Vaccine_Take_Age_Decay_Rate = Vaccine_Take_Age_Decay_Rate
        self.Vaccine_Type = Vaccine_Type.name if isinstance(Vaccine_Type, Enum) else Vaccine_Type
        self.Waning_Config = Waning_Config
        self.iv_type = iv_type



class BitingRisk(BaseCampaign):
    _definition = {'Constant': {'default': 6, 'depends-on': {'Risk_Distribution_Type': 'FIXED_DURATION'}, 'description': 'When Risk_Distribution_Type is set to FIXED_DURATION, this will specify the time delay (in number of days).', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Exponential_Mean': {'default': 6, 'depends-on': {'Risk_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'When Risk_Distribution_Type is set to EXPONENTIAL_DURATION, the Exponential_Mean parameter represents the exponential rate that describes the distribution of the time delay (in units of 1/days).', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Gaussian_Mean': {'default': 6, 'depends-on': {'Risk_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'When Risk_Distribution_Type is set to GAUSSIAN_DURATION, the Gaussian_Mean parameter defines the mean deviation of the Gaussian distribution for the active period. Negative values are truncated at zero.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Gaussian_Std_Dev': {'default': 1, 'depends-on': {'Risk_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'When Risk_Distribution_Type is set to GAUSSIAN_DURATION, the Gaussian_Std_Dev parameter defines the standard deviation of the Gaussian distribution for the active period. Negative values are truncated at zero.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'BitingRisk', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Risk_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The distribution of the time delay from when the biting risk intervention is distributed to when the individual actually receives the intervention.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'Uniform_Max': {'default': 1, 'depends-on': {'Risk_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum time delay (in number of days) when Risk_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Uniform_Min': {'default': 0, 'depends-on': {'Risk_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum time delay (in number of days) when Risk_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'class': 'BitingRisk', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'BitingRisk')

    def __init__(self, Constant=6, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Exponential_Mean=6, Gaussian_Mean=6, Gaussian_Std_Dev=1, Intervention_Name='BitingRisk', New_Property_Value=None, Risk_Distribution_Type=BitingRisk_Risk_Distribution_Type_Enum.NOT_INITIALIZED, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], Uniform_Max=1, Uniform_Min=0, iv_type='IndividualTargeted', **kwargs):
        super(BitingRisk, self).__init__(**kwargs)
        self.Constant = Constant
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Exponential_Mean = Exponential_Mean
        self.Gaussian_Mean = Gaussian_Mean
        self.Gaussian_Std_Dev = Gaussian_Std_Dev
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Risk_Distribution_Type = Risk_Distribution_Type.name if isinstance(Risk_Distribution_Type, Enum) else Risk_Distribution_Type
        self.Sim_Types = Sim_Types
        self.Uniform_Max = Uniform_Max
        self.Uniform_Min = Uniform_Min
        self.iv_type = iv_type



class BroadcastEvent(BaseCampaign):
    _definition = {'Broadcast_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event that should occur at the end of the delay period.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'BroadcastEvent', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'class': 'BroadcastEvent', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'BroadcastEvent')

    def __init__(self, Broadcast_Event='NoTrigger', Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='BroadcastEvent', New_Property_Value=None, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(BroadcastEvent, self).__init__(**kwargs)
        self.Broadcast_Event = Broadcast_Event
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class BroadcastEventToOtherNodes(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Trigger': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The name of the event to broadcast to selected nodes.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Include_My_Node': {'default': 0, 'description': 'Set to true (1) to broadcast the event to the current node.', 'type': 'bool'}, 'Intervention_Name': {'default': 'BroadcastEventToOtherNodes', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Max_Distance_To_Other_Nodes_Km': {'default': 3.40282e+38, 'description': 'The maximum distance, in kilometers, to the destination node for the node to be selected.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Node_Selection_Type': {'default': 'DISTANCE_ONLY', 'description': 'The method by which to select nodes to receive the event.', 'enum': ['DISTANCE_ONLY', 'MIGRATION_NODES_ONLY', 'DISTANCE_AND_MIGRATION'], 'type': 'enum'}, 'Sim_Types': ['*'], 'class': 'BroadcastEventToOtherNodes', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'BroadcastEventToOtherNodes')

    def __init__(self, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Trigger='NoTrigger', Include_My_Node=False, Intervention_Name='BroadcastEventToOtherNodes', Max_Distance_To_Other_Nodes_Km=3.40282e+38, New_Property_Value=None, Node_Selection_Type=BroadcastEventToOtherNodes_Node_Selection_Type_Enum.DISTANCE_ONLY, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(BroadcastEventToOtherNodes, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Trigger = Event_Trigger
        self.Include_My_Node = Include_My_Node
        self.Intervention_Name = Intervention_Name
        self.Max_Distance_To_Other_Nodes_Km = Max_Distance_To_Other_Nodes_Km
        self.New_Property_Value = New_Property_Value
        self.Node_Selection_Type = Node_Selection_Type.name if isinstance(Node_Selection_Type, Enum) else Node_Selection_Type
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class CD4Diagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'CD4_Thresholds': {'description': 'This parameter associates ranges of CD4 counts with events that should occur for individuals whose CD4 counts fall into those ranges.', 'type': 'list', 'item_type': {'type': 'object'}, 'default': []}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'CD4Diagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM', 'TBHIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'CD4Diagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'CD4Diagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, CD4_Thresholds=[], Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='CD4Diagnostic', New_Property_Value=None, Sim_Types=['HIV_SIM', 'TBHIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(CD4Diagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.CD4_Thresholds = CD4_Thresholds
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class ControlledVaccine(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Distributed_Event_Trigger': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The name of the event to be broadcast when the intervention is distributed to an individual. See the list of available events for possible values.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Duration_To_Wait_Before_Revaccination': {'default': 3.40282e+38, 'description': 'The length of time, in days, to wait before revaccinating an individual.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Efficacy_Is_Multiplicative': {'default': 1, 'description': 'The overall vaccine efficacy when individuals receive more than one vaccine. When set to true (1), the vaccine efficacies are multiplied together; when set to false (0), the efficacies are additive.', 'type': 'bool'}, 'Expired_Event_Trigger': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The name of the event to be broadcast when the intervention is distributed to an individual. See the list of available events for possible values.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Intervention_Name': {'default': 'ControlledVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'Vaccine_Take': {'default': 1, 'description': 'The rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.', 'max': 1, 'min': 0, 'type': 'float'}, 'Vaccine_Type': {'default': 'Generic', 'description': 'The type of vaccine to distribute in a vaccine intervention.', 'enum': ['Generic', 'TransmissionBlocking', 'AcquisitionBlocking', 'MortalityBlocking'], 'type': 'enum'}, 'Waning_Config': {'description': 'The configuration of Ivermectin killing efficacy and waning over time.', 'type': 'idmType:WaningEffect'}, 'class': 'ControlledVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ControlledVaccine')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Distributed_Event_Trigger='NoTrigger', Dont_Allow_Duplicates=False, Duration_To_Wait_Before_Revaccination=3.40282e+38, Efficacy_Is_Multiplicative=True, Expired_Event_Trigger='NoTrigger', Intervention_Name='ControlledVaccine', New_Property_Value=None, Sim_Types=['*'], Vaccine_Take=1, Vaccine_Type=ControlledVaccine_Vaccine_Type_Enum.Generic, Waning_Config=[], iv_type='IndividualTargeted', **kwargs):
        super(ControlledVaccine, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Distributed_Event_Trigger = Distributed_Event_Trigger
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Duration_To_Wait_Before_Revaccination = Duration_To_Wait_Before_Revaccination
        self.Efficacy_Is_Multiplicative = Efficacy_Is_Multiplicative
        self.Expired_Event_Trigger = Expired_Event_Trigger
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Vaccine_Take = Vaccine_Take
        self.Vaccine_Type = Vaccine_Type.name if isinstance(Vaccine_Type, Enum) else Vaccine_Type
        self.Waning_Config = Waning_Config
        self.iv_type = iv_type



class DelayedIntervention(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Configs': {'description': 'An array of nested interventions to be distributed at the end of a delay period, to covered fraction of the population.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Coverage': {'default': 1, 'description': 'The proportion of individuals who receive the DelayedIntervention that actually receive the configured interventions.', 'max': 1, 'min': 0, 'type': 'float'}, 'Delay_Distribution': {'default': 'NOT_INITIALIZED', 'description': 'The distribution of the time delay from when the intervention is distributed to when the individual actually receives the intervention.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Delay_Period': {'default': 6, 'depends-on': {'Delay_Distribution': 'EXPONENTIAL_DURATION'}, 'description': 'When Delay_Distribution is set to FIXED_DURATION, this will specify the time delay (in number of days). When Delay_Distribution is set to EXPONENTIAL_DURATION, this represents the exponential rate that describes the distribution of the time delay (in units of 1/days).', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Max': {'default': 1, 'depends-on': {'Delay_Distribution': 'UNIFORM_DURATION'}, 'description': 'The maximum time delay (in number of days) when Delay_Distribution is set to UNIFORM_DURATION. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Mean': {'default': 6, 'depends-on': {'Delay_Distribution': 'GAUSSIAN_DURATION'}, 'description': 'The mean time delay (in number of days), when Delay_Distribution is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Min': {'default': 0, 'depends-on': {'Delay_Distribution': 'UNIFORM_DURATION'}, 'description': 'The minimum time delay (in number of days) when Delay_Distribution is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Scale': {'default': 16, 'depends-on': {'Delay_Distribution': 'WEIBULL_DURATION'}, 'description': 'The scale parameter (lambda > 0) of the distribution (in days) when Delay_Distribution is set to WEIBULL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Shape': {'default': 20, 'depends-on': {'Delay_Distribution': 'WEIBULL_DURATION'}, 'description': 'The shape parameter (kappa > 0) of the distribution when Delay_Distribution is set to WEIBULL_DURATION. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Std_Dev': {'default': 1, 'depends-on': {'Delay_Distribution': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation of the Gaussian distribution (in number of days) when Delay_Distribution is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'DelayedIntervention', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'class': 'DelayedIntervention', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'DelayedIntervention')

    def __init__(self, Actual_IndividualIntervention_Configs=None, Coverage=1, Delay_Distribution=DelayedIntervention_Delay_Distribution_Enum.NOT_INITIALIZED, Delay_Period=6, Delay_Period_Max=1, Delay_Period_Mean=6, Delay_Period_Min=0, Delay_Period_Scale=16, Delay_Period_Shape=20, Delay_Period_Std_Dev=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='DelayedIntervention', New_Property_Value=None, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(DelayedIntervention, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Configs = Actual_IndividualIntervention_Configs
        self.Coverage = Coverage
        self.Delay_Distribution = Delay_Distribution.name if isinstance(Delay_Distribution, Enum) else Delay_Distribution
        self.Delay_Period = Delay_Period
        self.Delay_Period_Max = Delay_Period_Max
        self.Delay_Period_Mean = Delay_Period_Mean
        self.Delay_Period_Min = Delay_Period_Min
        self.Delay_Period_Scale = Delay_Period_Scale
        self.Delay_Period_Shape = Delay_Period_Shape
        self.Delay_Period_Std_Dev = Delay_Period_Std_Dev
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class DiagnosticTreatNeg(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Defaulters_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': "The intervention configuration given out when an individual did not return for test results when Event_Or_Config is set to 'Config'.", 'type': 'idmAbstractType:IndividualIntervention'}, 'Defaulters_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': "Specifies an event that can trigger another intervention when the event occurs. Event_Or_Config must be set to 'Event', and an individual did not return for test results. ", 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'DiagnosticTreatNeg', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'If Event_Or_Config is set to Config, this is the intervention given out when there is a negative diagnosis.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['TBHIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'DiagnosticTreatNeg', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'DiagnosticTreatNeg')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Defaulters_Config=None, Defaulters_Event='NoTrigger', Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=DiagnosticTreatNeg_Event_Or_Config_Enum.Config, Intervention_Name='DiagnosticTreatNeg', Negative_Diagnosis_Config=None, Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['TBHIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(DiagnosticTreatNeg, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Defaulters_Config = Defaulters_Config
        self.Defaulters_Event = Defaulters_Event
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Config = Negative_Diagnosis_Config
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVARTStagingByCD4Diagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'If_Active_TB': {'description': "If the individual's CD4 is not below the threshold in the Threshold table and the individual has TB (via their IndividualProperties), then the individual's CD4 will be compared to the CD4 value retrieved from the InterpolatedValueMap matrix based on the current year.", 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'If_Pregnant': {'description': "If the individual does not pass the diagnostic from the Threshold or TB matrices, and the individual is pregnant, then the individual's CD4 is compared to the value found in the InterpolatedValueMap matrix.", 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Individual_Property_Active_TB_Key': {'default': 'UNINITIALIZED', 'description': "The IndividualProperty key ('HasActiveTB') used to determine whether the individual has TB.", 'type': 'string'}, 'Individual_Property_Active_TB_Value': {'default': 'UNINITIALIZED', 'description': "The IndividualProperty value ('Yes') used to determine whether the individual has TB.", 'type': 'string'}, 'Intervention_Name': {'default': 'HIVARTStagingByCD4Diagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['HIV_SIM'], 'Threshold': {'description': "If the individual's CD4 has ever been below the threshold specified, then the test will be positive.", 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVARTStagingByCD4Diagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVARTStagingByCD4Diagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, If_Active_TB=None, If_Pregnant=None, Individual_Property_Active_TB_Key='UNINITIALIZED', Individual_Property_Active_TB_Value='UNINITIALIZED', Intervention_Name='HIVARTStagingByCD4Diagnostic', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['HIV_SIM'], Threshold=None, Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVARTStagingByCD4Diagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.If_Active_TB = If_Active_TB
        self.If_Pregnant = If_Pregnant
        self.Individual_Property_Active_TB_Key = Individual_Property_Active_TB_Key
        self.Individual_Property_Active_TB_Value = Individual_Property_Active_TB_Value
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Threshold = Threshold
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVARTStagingCD4AgnosticDiagnostic(BaseCampaign):
    _definition = {'Adult_By_Pregnant': {'description': 'Determines the WHO stage at or above which pregnant adults are eligible for ART.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Adult_By_TB': {'description': 'Determines the WHO stage at or above which adults having active TB (via individual property Has_Active_TB) are eligible for ART.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Adult_By_WHO_Stage': {'description': 'Determines the WHO stage at or above which adults are eligible for ART.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Adult_Treatment_Age': {'default': 5, 'description': 'The age that delineates adult patients from pediatric patients for the purpose of treatment eligibility.', 'max': 3.40282e+38, 'min': -1, 'type': 'float'}, 'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Child_By_TB': {'description': 'Determines the WHO stage at or above which children having active TB (via individual property Has_Active_TB) are eligible for ART.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Child_By_WHO_Stage': {'description': 'Determines the WHO stage at or above which children are eligible for ART.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Child_Treat_Under_Age_In_Years_Threshold': {'description': 'Determines the age at which children are eligible for ART regardless of CD4, WHO stage, or other factors.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Individual_Property_Active_TB_Key': {'default': 'UNINITIALIZED', 'description': "The IndividualProperty key ('HasActiveTB') used to determine whether the individual has TB.", 'type': 'string'}, 'Individual_Property_Active_TB_Value': {'default': 'UNINITIALIZED', 'description': "The IndividualProperty value ('Yes') used to determine whether the individual has TB.", 'type': 'string'}, 'Intervention_Name': {'default': 'HIVARTStagingCD4AgnosticDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['HIV_SIM', 'TBHIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVARTStagingCD4AgnosticDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVARTStagingCD4AgnosticDiagnostic')

    def __init__(self, Adult_By_Pregnant=None, Adult_By_TB=None, Adult_By_WHO_Stage=None, Adult_Treatment_Age=5, Base_Sensitivity=1, Base_Specificity=1, Child_By_TB=None, Child_By_WHO_Stage=None, Child_Treat_Under_Age_In_Years_Threshold=None, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Individual_Property_Active_TB_Key='UNINITIALIZED', Individual_Property_Active_TB_Value='UNINITIALIZED', Intervention_Name='HIVARTStagingCD4AgnosticDiagnostic', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['HIV_SIM', 'TBHIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVARTStagingCD4AgnosticDiagnostic, self).__init__(**kwargs)
        self.Adult_By_Pregnant = Adult_By_Pregnant
        self.Adult_By_TB = Adult_By_TB
        self.Adult_By_WHO_Stage = Adult_By_WHO_Stage
        self.Adult_Treatment_Age = Adult_Treatment_Age
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Child_By_TB = Child_By_TB
        self.Child_By_WHO_Stage = Child_By_WHO_Stage
        self.Child_Treat_Under_Age_In_Years_Threshold = Child_Treat_Under_Age_In_Years_Threshold
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Individual_Property_Active_TB_Key = Individual_Property_Active_TB_Key
        self.Individual_Property_Active_TB_Value = Individual_Property_Active_TB_Value
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVDelayedIntervention(BaseCampaign):
    _definition = {'Broadcast_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event that should occur at the end of the delay period.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Broadcast_On_Expiration_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the delay intervention expires before arriving at the end of the delay period, this specifies the event that should occur. ', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Coverage': {'default': 1, 'description': 'The proportion of individuals who receive the DelayedIntervention that actually receive the configured interventions.', 'max': 1, 'min': 0, 'type': 'float'}, 'Delay_Distribution': {'default': 'NOT_INITIALIZED', 'description': 'The distribution of the time delay from when the intervention is distributed to when the individual actually receives the intervention.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Delay_Period': {'default': 6, 'depends-on': {'Delay_Distribution': 'EXPONENTIAL_DURATION'}, 'description': 'When Delay_Distribution is set to FIXED_DURATION, this will specify the time delay (in number of days). When Delay_Distribution is set to EXPONENTIAL_DURATION, this represents the exponential rate that describes the distribution of the time delay (in units of 1/days).', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Max': {'default': 1, 'depends-on': {'Delay_Distribution': 'UNIFORM_DURATION'}, 'description': 'The maximum time delay (in number of days) when Delay_Distribution is set to UNIFORM_DURATION. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Mean': {'default': 6, 'depends-on': {'Delay_Distribution': 'GAUSSIAN_DURATION'}, 'description': 'The mean time delay (in number of days), when Delay_Distribution is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Min': {'default': 0, 'depends-on': {'Delay_Distribution': 'UNIFORM_DURATION'}, 'description': 'The minimum time delay (in number of days) when Delay_Distribution is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Scale': {'default': 16, 'depends-on': {'Delay_Distribution': 'WEIBULL_DURATION'}, 'description': 'The scale parameter (lambda > 0) of the distribution (in days) when Delay_Distribution is set to WEIBULL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Shape': {'default': 20, 'depends-on': {'Delay_Distribution': 'WEIBULL_DURATION'}, 'description': 'The shape parameter (kappa > 0) of the distribution when Delay_Distribution is set to WEIBULL_DURATION. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Std_Dev': {'default': 1, 'depends-on': {'Delay_Distribution': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation of the Gaussian distribution (in number of days) when Delay_Distribution is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Expiration_Period': {'default': 3.40282e+38, 'description': 'A fixed time period, in days, after which the Broadcast_On_Expiration_Event occurs instead of the Broadcast_Event. Only applied if the Expiration_Period occurs earlier than the end of the delay period.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'HIVDelayedIntervention', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM'], 'Time_Varying_Constants': {'depends-on': {'Delay_Distribution': 'PIECEWISE_CONSTANT || PIECEWISE_LINEAR'}, 'description': 'When Delay_Distribution is set to PIECEWISE_CONSTANT or PIECEWISE_LINEAR, this parameter maps simulation year to a delay period.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'class': 'HIVDelayedIntervention', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVDelayedIntervention')

    def __init__(self, Broadcast_Event='NoTrigger', Broadcast_On_Expiration_Event='NoTrigger', Coverage=1, Delay_Distribution=HIVDelayedIntervention_Delay_Distribution_Enum.NOT_INITIALIZED, Delay_Period=6, Delay_Period_Max=1, Delay_Period_Mean=6, Delay_Period_Min=0, Delay_Period_Scale=16, Delay_Period_Shape=20, Delay_Period_Std_Dev=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Expiration_Period=3.40282e+38, Intervention_Name='HIVDelayedIntervention', New_Property_Value=None, Sim_Types=['HIV_SIM'], Time_Varying_Constants=None, iv_type='IndividualTargeted', **kwargs):
        super(HIVDelayedIntervention, self).__init__(**kwargs)
        self.Broadcast_Event = Broadcast_Event
        self.Broadcast_On_Expiration_Event = Broadcast_On_Expiration_Event
        self.Coverage = Coverage
        self.Delay_Distribution = Delay_Distribution.name if isinstance(Delay_Distribution, Enum) else Delay_Distribution
        self.Delay_Period = Delay_Period
        self.Delay_Period_Max = Delay_Period_Max
        self.Delay_Period_Mean = Delay_Period_Mean
        self.Delay_Period_Min = Delay_Period_Min
        self.Delay_Period_Scale = Delay_Period_Scale
        self.Delay_Period_Shape = Delay_Period_Shape
        self.Delay_Period_Std_Dev = Delay_Period_Std_Dev
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Expiration_Period = Expiration_Period
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Time_Varying_Constants = Time_Varying_Constants
        self.iv_type = iv_type



class HIVDrawBlood(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'HIVDrawBlood', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVDrawBlood', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVDrawBlood')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='HIVDrawBlood', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVDrawBlood, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVMuxer(BaseCampaign):
    _definition = {'Broadcast_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event that should occur at the end of the delay period.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Broadcast_On_Expiration_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the delay intervention expires before arriving at the end of the delay period, this specifies the event that should occur. ', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Coverage': {'default': 1, 'description': 'The proportion of individuals who receive the DelayedIntervention that actually receive the configured interventions.', 'max': 1, 'min': 0, 'type': 'float'}, 'Delay_Distribution': {'default': 'NOT_INITIALIZED', 'description': 'The distribution of the time delay from when the intervention is distributed to when the individual actually receives the intervention.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Delay_Period': {'default': 6, 'depends-on': {'Delay_Distribution': 'EXPONENTIAL_DURATION'}, 'description': 'When Delay_Distribution is set to FIXED_DURATION, this will specify the time delay (in number of days). When Delay_Distribution is set to EXPONENTIAL_DURATION, this represents the exponential rate that describes the distribution of the time delay (in units of 1/days).', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Max': {'default': 1, 'depends-on': {'Delay_Distribution': 'UNIFORM_DURATION'}, 'description': 'The maximum time delay (in number of days) when Delay_Distribution is set to UNIFORM_DURATION. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Mean': {'default': 6, 'depends-on': {'Delay_Distribution': 'GAUSSIAN_DURATION'}, 'description': 'The mean time delay (in number of days), when Delay_Distribution is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Min': {'default': 0, 'depends-on': {'Delay_Distribution': 'UNIFORM_DURATION'}, 'description': 'The minimum time delay (in number of days) when Delay_Distribution is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Scale': {'default': 16, 'depends-on': {'Delay_Distribution': 'WEIBULL_DURATION'}, 'description': 'The scale parameter (lambda > 0) of the distribution (in days) when Delay_Distribution is set to WEIBULL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Shape': {'default': 20, 'depends-on': {'Delay_Distribution': 'WEIBULL_DURATION'}, 'description': 'The shape parameter (kappa > 0) of the distribution when Delay_Distribution is set to WEIBULL_DURATION. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Delay_Period_Std_Dev': {'default': 1, 'depends-on': {'Delay_Distribution': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation of the Gaussian distribution (in number of days) when Delay_Distribution is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Expiration_Period': {'default': 3.40282e+38, 'description': 'A fixed time period, in days, after which the Broadcast_On_Expiration_Event occurs instead of the Broadcast_Event. Only applied if the Expiration_Period occurs earlier than the end of the delay period.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'HIVMuxer', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Max_Entries': {'default': 1, 'description': 'The maximum number of times the individual can be registered with the HIVMuxer delay. Determines what should happen if an individual reaches the HIVMuxer stage of health care multiple times.', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'Muxer_Name': {'default': 'UNINITIALIZED STRING', 'description': 'A name used to identify the delay and check whether individuals have entered it multiple times. If the same name is used at multiple points in the health care process, then the number of entries is combined when Max_Entries is applied.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM'], 'Time_Varying_Constants': {'depends-on': {'Delay_Distribution': 'PIECEWISE_CONSTANT || PIECEWISE_LINEAR'}, 'description': 'When Delay_Distribution is set to PIECEWISE_CONSTANT or PIECEWISE_LINEAR, this parameter maps simulation year to a delay period.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'class': 'HIVMuxer', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVMuxer')

    def __init__(self, Broadcast_Event='NoTrigger', Broadcast_On_Expiration_Event='NoTrigger', Coverage=1, Delay_Distribution=HIVMuxer_Delay_Distribution_Enum.NOT_INITIALIZED, Delay_Period=6, Delay_Period_Max=1, Delay_Period_Mean=6, Delay_Period_Min=0, Delay_Period_Scale=16, Delay_Period_Shape=20, Delay_Period_Std_Dev=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Expiration_Period=3.40282e+38, Intervention_Name='HIVMuxer', Max_Entries=1, Muxer_Name='UNINITIALIZED STRING', New_Property_Value=None, Sim_Types=['HIV_SIM'], Time_Varying_Constants=None, iv_type='IndividualTargeted', **kwargs):
        super(HIVMuxer, self).__init__(**kwargs)
        self.Broadcast_Event = Broadcast_Event
        self.Broadcast_On_Expiration_Event = Broadcast_On_Expiration_Event
        self.Coverage = Coverage
        self.Delay_Distribution = Delay_Distribution.name if isinstance(Delay_Distribution, Enum) else Delay_Distribution
        self.Delay_Period = Delay_Period
        self.Delay_Period_Max = Delay_Period_Max
        self.Delay_Period_Mean = Delay_Period_Mean
        self.Delay_Period_Min = Delay_Period_Min
        self.Delay_Period_Scale = Delay_Period_Scale
        self.Delay_Period_Shape = Delay_Period_Shape
        self.Delay_Period_Std_Dev = Delay_Period_Std_Dev
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Expiration_Period = Expiration_Period
        self.Intervention_Name = Intervention_Name
        self.Max_Entries = Max_Entries
        self.Muxer_Name = Muxer_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Time_Varying_Constants = Time_Varying_Constants
        self.iv_type = iv_type



class HIVPiecewiseByYearAndSexDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Default_Value': {'default': 0, 'description': 'The probability of positive diagnosis if the intervention is used before the earliest specified time in the Time_Value_Map.', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Female_Multiplier': {'default': 1, 'description': 'Allows for the probabilities in the Time_Value_Map to be different for males and females, by multiplying the female probabilities by a constant value.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Interpolation_Order': {'default': 0, 'description': "When set to zero, interpolation between values in the Time_Value_Map is zero-order ('staircase'). When set to 1, interpolation between values in the Time_Value_Map is linear. The final value is held constant for all times after the last time specified in the Time_Value_Map.", 'max': 1, 'min': 0, 'type': 'integer'}, 'Intervention_Name': {'default': 'HIVPiecewiseByYearAndSexDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['HIV_SIM'], 'Time_Value_Map': {'description': 'The years (times) and matching probabilities for test results. ', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVPiecewiseByYearAndSexDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVPiecewiseByYearAndSexDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Default_Value=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Female_Multiplier=1, Interpolation_Order=0, Intervention_Name='HIVPiecewiseByYearAndSexDiagnostic', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['HIV_SIM'], Time_Value_Map=None, Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVPiecewiseByYearAndSexDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Default_Value = Default_Value
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Female_Multiplier = Female_Multiplier
        self.Interpolation_Order = Interpolation_Order
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Time_Value_Map = Time_Value_Map
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVRandomChoice(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Choices': {'description': 'An array of options for the individual and the probability of choosing each outcome. ', 'Event': {'description': 'Name of event to be broadcast if randomly selected.', 'type': 'Constrained String', 'value_source': '<configuration>::Listed_Events.*'}, 'Probability': {'description': 'The probability that the event will be selected. Values in map will be normalized.', 'max': 1, 'min': 0, 'type': 'float'}}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'HIVRandomChoice', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVRandomChoice', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVRandomChoice')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Choices=None, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='HIVRandomChoice', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVRandomChoice, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Choices = Choices
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVRapidHIVDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'HIVRapidHIVDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Probability_Received_Result': {'default': 1, 'description': 'The probability that an individual received the results of a diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Sim_Types': ['HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVRapidHIVDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVRapidHIVDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='HIVRapidHIVDiagnostic', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Probability_Received_Result=1, Sim_Types=['HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVRapidHIVDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Probability_Received_Result = Probability_Received_Result
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVSigmoidByYearAndSexDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Female_Multiplier': {'default': 1, 'description': 'Allows for the sigmoid time-varying probability to be different for males and females, by multiplying the female probability by a constant value.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'HIVSigmoidByYearAndSexDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Ramp_Max': {'default': 1, 'description': 'The right asymptote for the sigmoid trend over time.', 'max': 1, 'min': -1, 'type': 'float'}, 'Ramp_MidYear': {'default': 2000, 'description': 'The time of the infection point in the sigmoid trend over time.', 'max': 2200, 'min': 1900, 'type': 'float'}, 'Ramp_Min': {'default': 0, 'description': 'The left asymptote for the sigmoid trend over time.', 'max': 1, 'min': -1, 'type': 'float'}, 'Ramp_Rate': {'default': 1, 'description': 'The slope of the inflection point in the sigmoid trend over time. A Rate of 1 sets the slope to a 25% change in probability per year.', 'max': 100, 'min': -100, 'type': 'float'}, 'Sim_Types': ['HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVSigmoidByYearAndSexDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVSigmoidByYearAndSexDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Female_Multiplier=1, Intervention_Name='HIVSigmoidByYearAndSexDiagnostic', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Ramp_Max=1, Ramp_MidYear=2000, Ramp_Min=0, Ramp_Rate=1, Sim_Types=['HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVSigmoidByYearAndSexDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Female_Multiplier = Female_Multiplier
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Ramp_Max = Ramp_Max
        self.Ramp_MidYear = Ramp_MidYear
        self.Ramp_Min = Ramp_Min
        self.Ramp_Rate = Ramp_Rate
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HIVSimpleDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'HIVSimpleDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['HIV_SIM', 'TBHIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HIVSimpleDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HIVSimpleDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='HIVSimpleDiagnostic', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['HIV_SIM', 'TBHIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(HIVSimpleDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class HealthSeekingBehaviorUpdate(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'HealthSeekingBehaviorUpdate', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'New_Tendency': {'default': 1, 'description': 'Updates the tendency specified by HealthSeekingBehaviorUpdateable.', 'max': 3.40282e+38, 'min': -3.40282e+38, 'type': 'float'}, 'Sim_Types': [], 'class': 'HealthSeekingBehaviorUpdate', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HealthSeekingBehaviorUpdate')

    def __init__(self, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='HealthSeekingBehaviorUpdate', New_Property_Value=None, New_Tendency=1, Sim_Types=[], iv_type='IndividualTargeted', **kwargs):
        super(HealthSeekingBehaviorUpdate, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.New_Tendency = New_Tendency
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class HealthSeekingBehaviorUpdateable(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The configuration of an actual intervention sought. Selects a class for the intervention and configures the parameters specific for that intervention class.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Actual_IndividualIntervention_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event of an actual intervention sought. Selects a class for the intervention and configures the parameters specific for that intervention class. See the list of available events for possible values.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'HealthSeekingBehaviorUpdateable', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': [], 'Single_Use': {'default': 1, 'description': 'If set to true (1), the health-seeking behavior gets used once and discarded. If set to false (0), it remains indefinitely.', 'type': 'bool'}, 'Tendency': {'default': 1, 'description': 'The probability of seeking healthcare.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'HealthSeekingBehaviorUpdateable', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HealthSeekingBehaviorUpdateable')

    def __init__(self, Actual_IndividualIntervention_Config=None, Actual_IndividualIntervention_Event='NoTrigger', Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=HealthSeekingBehaviorUpdateable_Event_Or_Config_Enum.Config, Intervention_Name='HealthSeekingBehaviorUpdateable', New_Property_Value=None, Sim_Types=[], Single_Use=True, Tendency=1, iv_type='IndividualTargeted', **kwargs):
        super(HealthSeekingBehaviorUpdateable, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Config = Actual_IndividualIntervention_Config
        self.Actual_IndividualIntervention_Event = Actual_IndividualIntervention_Event
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Single_Use = Single_Use
        self.Tendency = Tendency
        self.iv_type = iv_type



class HumanHostSeekingTrap(BaseCampaign):
    _definition = {'Attract_Config': {'description': 'The configuration of attraction efficacy and waning for human host-seeking trap. Decays over time.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 3.75, 'description': 'Unit cost per trap (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'HumanHostSeekingTrap', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for human host-seeking trap.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'HumanHostSeekingTrap', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'HumanHostSeekingTrap')

    def __init__(self, Attract_Config=None, Cost_To_Consumer=3.75, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='HumanHostSeekingTrap', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(HumanHostSeekingTrap, self).__init__(**kwargs)
        self.Attract_Config = Attract_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class IRSHousingModification(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of pre-feed mosquito repellency and waning for housing modification.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per housing modification (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'IRSHousingModification', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for housing modification. Killing is conditional on NOT blocking the mosquito before feeding.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'IRSHousingModification', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'IRSHousingModification')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='IRSHousingModification', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(IRSHousingModification, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class IVCalendar(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Configs': {'description': 'An array of interventions that will be distributed as specified in the calendar. ', 'type': 'idmAbstractType:IndividualIntervention'}, 'Calendar': {'description': 'An array of ages, days and the probabilities of receiving the list of interventions at each age.', 'type': 'list', 'item_type': {'type': 'object'}, 'default': []}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Dropout': {'default': 0, 'description': 'If set to true (1), when an intervention distribution is missed, all subsequent interventions are also missed. If set to false (0), all calendar dates/doses are applied independently of each other.', 'type': 'bool'}, 'Intervention_Name': {'default': 'IVCalendar', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'class': 'IVCalendar', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'IVCalendar')

    def __init__(self, Actual_IndividualIntervention_Configs=None, Calendar=[], Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Dropout=False, Intervention_Name='IVCalendar', New_Property_Value=None, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(IVCalendar, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Configs = Actual_IndividualIntervention_Configs
        self.Calendar = Calendar
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Dropout = Dropout
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class IndividualImmunityChanger(BaseCampaign):
    _definition = {'Boost_Acquire': {'default': 0, 'description': 'Specifies the boosting effect on acquisition immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine. ', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Mortality': {'default': 0, 'description': 'Specifies the boosting effect on mortality immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold_Acquire': {'default': 0, 'description': 'Specifies how much acquisition immunity is required before the vaccine changes from a prime to a boost.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold_Mortality': {'default': 0, 'description': 'Specifies how much mortality immunity is required before the vaccine changes from a prime to a boost.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold_Transmit': {'default': 0, 'description': 'Specifies how much transmission immunity is required before the vaccine changes from a prime to a boost.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Transmit': {'default': 0, 'description': 'Specifies the boosting effect on transmission immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine. ', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Prime_Acquire': {'default': 0, 'description': 'Specifies the priming effect on acquisition immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Prime_Mortality': {'default': 0, 'description': 'Specifies the priming effect on mortality immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Prime_Transmit': {'default': 0, 'description': 'Specifies the priming effect on transmission immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Sim_Types': ['*'], 'class': 'IndividualImmunityChanger', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'IndividualImmunityChanger')

    def __init__(self, Boost_Acquire=0, Boost_Mortality=0, Boost_Threshold_Acquire=0, Boost_Threshold_Mortality=0, Boost_Threshold_Transmit=0, Boost_Transmit=0, Cost_To_Consumer=1, Prime_Acquire=0, Prime_Mortality=0, Prime_Transmit=0, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(IndividualImmunityChanger, self).__init__(**kwargs)
        self.Boost_Acquire = Boost_Acquire
        self.Boost_Mortality = Boost_Mortality
        self.Boost_Threshold_Acquire = Boost_Threshold_Acquire
        self.Boost_Threshold_Mortality = Boost_Threshold_Mortality
        self.Boost_Threshold_Transmit = Boost_Threshold_Transmit
        self.Boost_Transmit = Boost_Transmit
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Prime_Acquire = Prime_Acquire
        self.Prime_Mortality = Prime_Mortality
        self.Prime_Transmit = Prime_Transmit
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class InsectKillingFenceHousingModification(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of pre-feed mosquito repellency and waning for housing modification.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per housing modification (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'InsectKillingFenceHousingModification', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for housing modification. Killing is conditional on NOT blocking the mosquito before feeding.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'InsectKillingFenceHousingModification', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'InsectKillingFenceHousingModification')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='InsectKillingFenceHousingModification', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(InsectKillingFenceHousingModification, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class Ivermectin(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per Ivermectin dosing (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'Ivermectin', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of Ivermectin killing efficacy and waning over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'Ivermectin', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'Ivermectin')

    def __init__(self, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='Ivermectin', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(Ivermectin, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class MDRDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Defaulters_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': "The intervention configuration given out when an individual did not return for test results when Event_Or_Config is set to 'Config'.", 'type': 'idmAbstractType:IndividualIntervention'}, 'Defaulters_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': "Specifies an event that can trigger another intervention when the event occurs. Event_Or_Config must be set to 'Event', and an individual did not return for test results. ", 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'MDRDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'If Event_Or_Config is set to Config, this is the intervention given out when there is a negative diagnosis.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If an individual tests negative, this specifies an event that may trigger another intervention when the event occurs.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['TBHIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'Treatment_Fraction_Negative_Diagnosis': {'default': 1, 'description': 'The fraction of negative diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'MDRDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MDRDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Defaulters_Config=None, Defaulters_Event='NoTrigger', Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=MDRDiagnostic_Event_Or_Config_Enum.Config, Intervention_Name='MDRDiagnostic', Negative_Diagnosis_Config=None, Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['TBHIV_SIM'], Treatment_Fraction=1, Treatment_Fraction_Negative_Diagnosis=1, iv_type='IndividualTargeted', **kwargs):
        super(MDRDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Defaulters_Config = Defaulters_Config
        self.Defaulters_Event = Defaulters_Event
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Config = Negative_Diagnosis_Config
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.Treatment_Fraction_Negative_Diagnosis = Treatment_Fraction_Negative_Diagnosis
        self.iv_type = iv_type



class MalariaDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Detection_Threshold': {'default': 100, 'depends-on': {'Diagnostic_Type': 'Other'}, 'description': 'The diagnostic detection threshold for parasites, in units of microliters of blood.', 'max': 1000000.0, 'min': 0, 'type': 'float'}, 'Diagnostic_Type': {'default': 'Microscopy', 'description': 'The type of malaria diagnostic used.', 'enum': ['Microscopy', 'NewDetectionTech', 'Other'], 'type': 'enum'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'MalariaDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['MALARIA_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'MalariaDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MalariaDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Detection_Threshold=100, Diagnostic_Type=MalariaDiagnostic_Diagnostic_Type_Enum.Microscopy, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=MalariaDiagnostic_Event_Or_Config_Enum.Config, Intervention_Name='MalariaDiagnostic', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['MALARIA_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(MalariaDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Detection_Threshold = Detection_Threshold
        self.Diagnostic_Type = Diagnostic_Type.name if isinstance(Diagnostic_Type, Enum) else Diagnostic_Type
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class MaleCircumcision(BaseCampaign):
    _definition = {'Apply_If_Higher_Reduced_Acquire': {'default': 0, 'description': 'If set to false (0), the MaleCircumcision intervention can never be applied to someone who already has a MaleCircumcision intervention. If set to true (1), a male who already has a MaleCircumcision intervention, but whose pre-existing MaleCircumcision intervention has a lower efficacy parameter (Circumcision_Reduced_Acquire) than the one about to be applied, will receive the higher-efficacy MaleCircumcision.', 'type': 'bool'}, 'Circumcision_Reduced_Acquire': {'default': 0.6, 'description': 'The reduction of susceptibility to STI by voluntary male medical circumcision (VMMC).', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Distributed_Event_Trigger': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'When defined as part of an intervention block of class MaleCircumcision, this string defines the name of the column in the output files ReportHIVByAgeAndGender.csv and ReportEventRecorder.csv, which log when the intervention has been distributed.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'MaleCircumcision', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['STI_SIM', 'HIV_SIM'], 'class': 'MaleCircumcision', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MaleCircumcision')

    def __init__(self, Apply_If_Higher_Reduced_Acquire=False, Circumcision_Reduced_Acquire=0.6, Disqualifying_Properties=[], Distributed_Event_Trigger='NoTrigger', Dont_Allow_Duplicates=False, Intervention_Name='MaleCircumcision', New_Property_Value=None, Sim_Types=['STI_SIM', 'HIV_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(MaleCircumcision, self).__init__(**kwargs)
        self.Apply_If_Higher_Reduced_Acquire = Apply_If_Higher_Reduced_Acquire
        self.Circumcision_Reduced_Acquire = Circumcision_Reduced_Acquire
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Distributed_Event_Trigger = Distributed_Event_Trigger
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class MigrateIndividuals(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Duration_At_Node_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The shape of the distribution for the amount of time spent at the destination node for intervention-based migration.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Duration_At_Node_Exponential_Period': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'The period (1/rate) to use for an exponential distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to EXPONENTIAL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Fixed': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'FIXED_DURATION'}, 'description': 'The value to use for a fixed distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to FIXED_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Gausian_Mean': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The mean value to use for a Gaussian distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Gausian_StdDev': {'default': 1, 'depends-on': {'Duration_At_Node_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation to use for a Gaussian distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Poisson_Mean': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'POISSON_DURATION'}, 'description': 'The mean to use for a Poisson distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to POISSON_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Uniform_Max': {'default': 1, 'depends-on': {'Duration_At_Node_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum value to use for a uniform distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Uniform_Min': {'default': 0, 'depends-on': {'Duration_At_Node_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum value to use for a uniform distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The shape of the distribution of the number of days to wait before migrating to the destination node for intervention-based migration. ', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Duration_Before_Leaving_Exponential_Period': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'The period (1/rate) to use for an exponential distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to EXPONENTIAL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Fixed': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'FIXED_DURATION'}, 'description': 'The value to use for a fixed distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to FIXED_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Gausian_Mean': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The mean value to use for a Gaussian distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Gausian_StdDev': {'default': 1, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation to use for a Gaussian distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Poisson_Mean': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'POISSON_DURATION'}, 'description': 'The mean to use for a Poisson distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to POISSON_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Uniform_Max': {'default': 1, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum value to use for a uniform distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Uniform_Min': {'default': 0, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum value to use for a uniform distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'MigrateIndividuals', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Is_Moving': {'default': 0, 'description': 'Set to true (1) to indicate the individual is permanently moving to a new home node for intervention-based migration.', 'type': 'bool'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'NodeID_To_Migrate_To': {'default': 0, 'description': 'The destination node ID for intervention-based migration.', 'max': 4294970000.0, 'min': 0, 'type': 'integer'}, 'Sim_Types': ['*'], 'class': 'MigrateIndividuals', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MigrateIndividuals')

    def __init__(self, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Duration_At_Node_Distribution_Type=MigrateIndividuals_Duration_At_Node_Distribution_Type_Enum.NOT_INITIALIZED, Duration_At_Node_Exponential_Period=6, Duration_At_Node_Fixed=6, Duration_At_Node_Gausian_Mean=6, Duration_At_Node_Gausian_StdDev=1, Duration_At_Node_Poisson_Mean=6, Duration_At_Node_Uniform_Max=1, Duration_At_Node_Uniform_Min=0, Duration_Before_Leaving_Distribution_Type=MigrateIndividuals_Duration_Before_Leaving_Distribution_Type_Enum.NOT_INITIALIZED, Duration_Before_Leaving_Exponential_Period=6, Duration_Before_Leaving_Fixed=6, Duration_Before_Leaving_Gausian_Mean=6, Duration_Before_Leaving_Gausian_StdDev=1, Duration_Before_Leaving_Poisson_Mean=6, Duration_Before_Leaving_Uniform_Max=1, Duration_Before_Leaving_Uniform_Min=0, Intervention_Name='MigrateIndividuals', Is_Moving=False, New_Property_Value=None, NodeID_To_Migrate_To=0, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(MigrateIndividuals, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Duration_At_Node_Distribution_Type = Duration_At_Node_Distribution_Type.name if isinstance(Duration_At_Node_Distribution_Type, Enum) else Duration_At_Node_Distribution_Type
        self.Duration_At_Node_Exponential_Period = Duration_At_Node_Exponential_Period
        self.Duration_At_Node_Fixed = Duration_At_Node_Fixed
        self.Duration_At_Node_Gausian_Mean = Duration_At_Node_Gausian_Mean
        self.Duration_At_Node_Gausian_StdDev = Duration_At_Node_Gausian_StdDev
        self.Duration_At_Node_Poisson_Mean = Duration_At_Node_Poisson_Mean
        self.Duration_At_Node_Uniform_Max = Duration_At_Node_Uniform_Max
        self.Duration_At_Node_Uniform_Min = Duration_At_Node_Uniform_Min
        self.Duration_Before_Leaving_Distribution_Type = Duration_Before_Leaving_Distribution_Type.name if isinstance(Duration_Before_Leaving_Distribution_Type, Enum) else Duration_Before_Leaving_Distribution_Type
        self.Duration_Before_Leaving_Exponential_Period = Duration_Before_Leaving_Exponential_Period
        self.Duration_Before_Leaving_Fixed = Duration_Before_Leaving_Fixed
        self.Duration_Before_Leaving_Gausian_Mean = Duration_Before_Leaving_Gausian_Mean
        self.Duration_Before_Leaving_Gausian_StdDev = Duration_Before_Leaving_Gausian_StdDev
        self.Duration_Before_Leaving_Poisson_Mean = Duration_Before_Leaving_Poisson_Mean
        self.Duration_Before_Leaving_Uniform_Max = Duration_Before_Leaving_Uniform_Max
        self.Duration_Before_Leaving_Uniform_Min = Duration_Before_Leaving_Uniform_Min
        self.Intervention_Name = Intervention_Name
        self.Is_Moving = Is_Moving
        self.New_Property_Value = New_Property_Value
        self.NodeID_To_Migrate_To = NodeID_To_Migrate_To
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class ModifyStiCoInfectionStatus(BaseCampaign):
    _definition = {'New_STI_CoInfection_Status': {'default': 0, 'description': 'Determines whether to apply STI co-infection, or cure/remove STI co-infection. Set to true (1) to include co-infection; set to false (0) to remove co-infection.', 'type': 'bool'}, 'Sim_Types': ['STI_SIM', 'HIV_SIM'], 'class': 'ModifyStiCoInfectionStatus', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ModifyStiCoInfectionStatus')

    def __init__(self, New_STI_CoInfection_Status=False, Sim_Types=['STI_SIM', 'HIV_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(ModifyStiCoInfectionStatus, self).__init__(**kwargs)
        self.New_STI_CoInfection_Status = New_STI_CoInfection_Status
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class MultiEffectBoosterVaccine(BaseCampaign):
    _definition = {'Acquire_Config': {'description': 'The configuration for multi-effect vaccine acquisition.', 'type': 'idmType:WaningEffect'}, 'Boost_Acquire': {'default': 0, 'description': 'Specifies the boosting effect on acquisition immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine. ', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Mortality': {'default': 0, 'description': 'Specifies the boosting effect on mortality immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold_Acquire': {'default': 0, 'description': 'Specifies how much acquisition immunity is required before the vaccine changes from a prime to a boost.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold_Mortality': {'default': 0, 'description': 'Specifies how much mortality immunity is required before the vaccine changes from a prime to a boost.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold_Transmit': {'default': 0, 'description': 'Specifies how much transmission immunity is required before the vaccine changes from a prime to a boost.', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Transmit': {'default': 0, 'description': 'Specifies the boosting effect on transmission immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine. ', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'MultiEffectBoosterVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Mortality_Config': {'description': 'The configuration for multi-effect vaccine mortality.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Prime_Acquire': {'default': 0, 'description': 'Specifies the priming effect on acquisition immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Prime_Mortality': {'default': 0, 'description': 'Specifies the priming effect on mortality immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Prime_Transmit': {'default': 0, 'description': 'Specifies the priming effect on transmission immunity for naive individuals (without natural or vaccine-derived immunity) for a multi-effect booster vaccine.', 'max': 1, 'min': 0, 'type': 'float'}, 'Sim_Types': ['*'], 'Transmit_Config': {'description': 'The configuration for multi-effect vaccine transmission.', 'type': 'idmType:WaningEffect'}, 'Vaccine_Take': {'default': 1, 'description': 'The rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'MultiEffectBoosterVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MultiEffectBoosterVaccine')

    def __init__(self, Acquire_Config=None, Boost_Acquire=0, Boost_Mortality=0, Boost_Threshold_Acquire=0, Boost_Threshold_Mortality=0, Boost_Threshold_Transmit=0, Boost_Transmit=0, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='MultiEffectBoosterVaccine', Mortality_Config=None, New_Property_Value=None, Prime_Acquire=0, Prime_Mortality=0, Prime_Transmit=0, Sim_Types=['*'], Transmit_Config=None, Vaccine_Take=1, iv_type='IndividualTargeted', **kwargs):
        super(MultiEffectBoosterVaccine, self).__init__(**kwargs)
        self.Acquire_Config = Acquire_Config
        self.Boost_Acquire = Boost_Acquire
        self.Boost_Mortality = Boost_Mortality
        self.Boost_Threshold_Acquire = Boost_Threshold_Acquire
        self.Boost_Threshold_Mortality = Boost_Threshold_Mortality
        self.Boost_Threshold_Transmit = Boost_Threshold_Transmit
        self.Boost_Transmit = Boost_Transmit
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Mortality_Config = Mortality_Config
        self.New_Property_Value = New_Property_Value
        self.Prime_Acquire = Prime_Acquire
        self.Prime_Mortality = Prime_Mortality
        self.Prime_Transmit = Prime_Transmit
        self.Sim_Types = Sim_Types
        self.Transmit_Config = Transmit_Config
        self.Vaccine_Take = Vaccine_Take
        self.iv_type = iv_type



class MultiEffectVaccine(BaseCampaign):
    _definition = {'Acquire_Config': {'description': 'The configuration for multi-effect vaccine acquisition.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'MultiEffectVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Mortality_Config': {'description': 'The configuration for multi-effect vaccine mortality.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'Transmit_Config': {'description': 'The configuration for multi-effect vaccine transmission.', 'type': 'idmType:WaningEffect'}, 'Vaccine_Take': {'default': 1, 'description': 'The rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'MultiEffectVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MultiEffectVaccine')

    def __init__(self, Acquire_Config=None, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='MultiEffectVaccine', Mortality_Config=None, New_Property_Value=None, Sim_Types=['*'], Transmit_Config=None, Vaccine_Take=1, iv_type='IndividualTargeted', **kwargs):
        super(MultiEffectVaccine, self).__init__(**kwargs)
        self.Acquire_Config = Acquire_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Mortality_Config = Mortality_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Transmit_Config = Transmit_Config
        self.Vaccine_Take = Vaccine_Take
        self.iv_type = iv_type



class MultiInterventionDistributor(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_List': {'description': 'The list of individual interventions that is distributed by MultiInterventionDistributor.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Intervention_Name': {'default': 'MultiInterventionDistributor', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'class': 'MultiInterventionDistributor', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MultiInterventionDistributor')

    def __init__(self, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_List=None, Intervention_Name='MultiInterventionDistributor', New_Property_Value=None, Sim_Types=['*'], iv_type='IndividualTargeted', **kwargs):
        super(MultiInterventionDistributor, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_List = Intervention_List
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class OutbreakIndividual(BaseCampaign):
    _definition = {'Antigen': {'default': 0, 'description': 'The antigenic ID of the outbreak infection.', 'max': 10, 'min': 0, 'type': 'integer'}, 'Genome': {'default': 0, 'description': 'The genetic ID of the outbreak infection.', 'max': 16777200.0, 'min': -1, 'type': 'integer'}, 'Ignore_Immunity': {'default': 1, 'description': 'Individuals will be force-infected (with a specific strain) regardless of actual immunity level when set to true (1).', 'type': 'bool'}, 'Incubation_Period_Override': {'default': -1, 'description': 'The incubation period, in days, that infected individuals will go through before becoming infectious. This value overrides the incubation period set in the configuration file. Set to -1 to honor the configuration parameter settings.', 'max': 2147480000.0, 'min': -1, 'type': 'integer'}, 'Sim_Types': ['GENERIC_SIM', 'VECTOR_SIM', 'MALARIA_SIM', 'AIRBORNE_SIM', 'POLIO_SIM', 'TBHIV_SIM', 'STI_SIM', 'HIV_SIM', 'PY_SIM', 'TYPHOID_SIM'], 'class': 'OutbreakIndividual', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'OutbreakIndividual')

    def __init__(self, Antigen=0, Genome=0, Ignore_Immunity=True, Incubation_Period_Override=-1, Sim_Types=['GENERIC_SIM', 'VECTOR_SIM', 'MALARIA_SIM', 'AIRBORNE_SIM', 'POLIO_SIM', 'TBHIV_SIM', 'STI_SIM', 'HIV_SIM', 'PY_SIM', 'TYPHOID_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(OutbreakIndividual, self).__init__(**kwargs)
        self.Antigen = Antigen
        self.Genome = Genome
        self.Ignore_Immunity = Ignore_Immunity
        self.Incubation_Period_Override = Incubation_Period_Override
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class OutbreakIndividualMalaria(BaseCampaign):
    _definition = {'Antigen': {'default': 0, 'description': 'The antigenic ID of the outbreak infection.', 'max': 10, 'min': 0, 'type': 'integer'}, 'Create_Random_Genome': {'default': 0, 'description': 'If set to true (1), then a random genome is created for the infection and the Genome_Markers parameter is not used. If set to false (0), then you must define the  Genome_Markers parameter which allows you to then specify genetic components in a strain of infection.', 'type': 'bool'}, 'Genome_Markers': {'description': 'A list of the names of genome marker(s) that represent the genetic components in a strain of an infection.', 'type': 'Vector String', 'value_source': '<configuration>.Genome_Markers'}, 'Ignore_Immunity': {'default': 1, 'description': 'Individuals will be force-infected (with a specific strain) regardless of actual immunity level when set to true (1).', 'type': 'bool'}, 'Incubation_Period_Override': {'default': -1, 'description': 'The incubation period, in days, that infected individuals will go through before becoming infectious. This value overrides the incubation period set in the configuration file. Set to -1 to honor the configuration parameter settings.', 'max': 2147480000.0, 'min': -1, 'type': 'integer'}, 'Sim_Types': ['MALARIA_SIM'], 'class': 'OutbreakIndividualMalaria', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'OutbreakIndividualMalaria')

    def __init__(self, Antigen=0, Create_Random_Genome=False, Genome_Markers=None, Ignore_Immunity=True, Incubation_Period_Override=-1, Sim_Types=['MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(OutbreakIndividualMalaria, self).__init__(**kwargs)
        self.Antigen = Antigen
        self.Create_Random_Genome = Create_Random_Genome
        self.Genome_Markers = Genome_Markers
        self.Ignore_Immunity = Ignore_Immunity
        self.Incubation_Period_Override = Incubation_Period_Override
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class PMTCT(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Efficacy': {'default': 0.5, 'description': 'Represents the efficacy of a Prevention of Mother to Child Transmission (PMTCT) intervention, defined as the rate ratio of mother to child transmission (MTCT) between women receiving the intervention and women not receiving the intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'PMTCT', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['HIV_SIM'], 'class': 'PMTCT', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'PMTCT')

    def __init__(self, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Efficacy=0.5, Intervention_Name='PMTCT', New_Property_Value=None, Sim_Types=['HIV_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(PMTCT, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Efficacy = Efficacy
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class PropertyValueChanger(BaseCampaign):
    _definition = {'Daily_Probability': {'default': 1, 'description': 'The probability that an individual will move to the Target_Property_Value.', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'PropertyValueChanger', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Maximum_Duration': {'default': 3.40282e+38, 'description': 'The maximum amount of time individuals have to move to a new group. This timing works in conjunction with Daily_Probability.', 'max': 3.40282e+38, 'min': -1, 'type': 'float'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Revert': {'default': 0, 'description': 'The number of days before an individual moves back to their original group.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Sim_Types': ['*'], 'Target_Property_Key': {'default': 'UNINITIALIZED STRING', 'description': 'The name of the individual property type whose value will be updated by the intervention.', 'type': 'Constrained String', 'value_source': '<demographics>::*.Individual_Properties.*.Property'}, 'Target_Property_Value': {'default': 'UNINITIALIZED STRING', 'description': 'The user-defined value of the individual property that will be assigned to the individual.', 'type': 'Constrained String', 'value_source': '<demographics>::*.Individual_Properties.*.Values'}, 'class': 'PropertyValueChanger', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'PropertyValueChanger')

    def __init__(self, Daily_Probability=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='PropertyValueChanger', Maximum_Duration=3.40282e+38, New_Property_Value=None, Revert=0, Sim_Types=['*'], Target_Property_Key='UNINITIALIZED STRING', Target_Property_Value='UNINITIALIZED STRING', iv_type='IndividualTargeted', **kwargs):
        super(PropertyValueChanger, self).__init__(**kwargs)
        self.Daily_Probability = Daily_Probability
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Maximum_Duration = Maximum_Duration
        self.New_Property_Value = New_Property_Value
        self.Revert = Revert
        self.Sim_Types = Sim_Types
        self.Target_Property_Key = Target_Property_Key
        self.Target_Property_Value = Target_Property_Value
        self.iv_type = iv_type



class RTSSVaccine(BaseCampaign):
    _definition = {'Antibody_Type': {'default': 'CSP', 'description': 'The antibody type used to determine immune response targets.', 'enum': ['CSP', 'MSP1', 'PfEMP1_minor', 'PfEMP1_major', 'N_MALARIA_ANTIBODY_TYPES'], 'type': 'enum'}, 'Antibody_Variant': {'default': 0, 'description': 'The antibody variant index.', 'max': 100000, 'min': 0, 'type': 'integer'}, 'Boosted_Antibody_Concentration': {'default': 1, 'description': 'The boosted antibody concentration, where unity equals maximum from natural exposure.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 3.75, 'description': 'Unit cost of RTS,S vaccination (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'RTSSVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['MALARIA_SIM'], 'class': 'RTSSVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'RTSSVaccine')

    def __init__(self, Antibody_Type=RTSSVaccine_Antibody_Type_Enum.CSP, Antibody_Variant=0, Boosted_Antibody_Concentration=1, Cost_To_Consumer=3.75, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='RTSSVaccine', New_Property_Value=None, Sim_Types=['MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(RTSSVaccine, self).__init__(**kwargs)
        self.Antibody_Type = Antibody_Type.name if isinstance(Antibody_Type, Enum) else Antibody_Type
        self.Antibody_Variant = Antibody_Variant
        self.Boosted_Antibody_Concentration = Boosted_Antibody_Concentration
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class STIBarrier(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Determines the unit cost when using the STIBarrier intervention to change defaults from demographics. Note that there is no cost for condoms distributed using demographics-configured default usage probabilities.', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Early': {'default': 1, 'description': 'The left asymptote for the sigmoid trend over time. The Early value must be smaller than the Late value.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'STIBarrier', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Late': {'default': 1, 'description': 'The right asymptote for the sigmoid trend over time. The Late value must be larger than the Early value.', 'max': 1, 'min': 0, 'type': 'float'}, 'MidYear': {'default': 2000, 'description': 'The time of the inflection point in the sigmoid trend over time.', 'max': 2200, 'min': 1900, 'type': 'float'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Rate': {'default': 1, 'description': 'The slope of the inflection point in the sigmoid trend over time. A Rate of 1 sets the slope to a 25% change in probability per year. Specify a negative Rate (e.g. -1) to achieve a negative sigmoid.', 'max': 100, 'min': -100, 'type': 'float'}, 'Relationship_Type': {'default': 'TRANSITORY', 'description': 'The relationship type to which the condom usage probability is applied.', 'enum': ['TRANSITORY', 'INFORMAL', 'MARITAL', 'COMMERCIAL', 'COUNT'], 'type': 'enum'}, 'Sim_Types': ['STI_SIM', 'HIV_SIM'], 'class': 'STIBarrier', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'STIBarrier')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Early=1, Intervention_Name='STIBarrier', Late=1, MidYear=2000, New_Property_Value=None, Rate=1, Relationship_Type=STIBarrier_Relationship_Type_Enum.TRANSITORY, Sim_Types=['STI_SIM', 'HIV_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(STIBarrier, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Early = Early
        self.Intervention_Name = Intervention_Name
        self.Late = Late
        self.MidYear = MidYear
        self.New_Property_Value = New_Property_Value
        self.Rate = Rate
        self.Relationship_Type = Relationship_Type.name if isinstance(Relationship_Type, Enum) else Relationship_Type
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class STIIsPostDebut(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'STIIsPostDebut', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Negative_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The name of the event to broadcast when an individual is found to NOT be Post-Debut age.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['STI_SIM', 'HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'STIIsPostDebut', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'STIIsPostDebut')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=STIIsPostDebut_Event_Or_Config_Enum.Config, Intervention_Name='STIIsPostDebut', Negative_Diagnosis_Event='NoTrigger', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['STI_SIM', 'HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(STIIsPostDebut, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.Negative_Diagnosis_Event = Negative_Diagnosis_Event
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class ScreeningHousingModification(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of pre-feed mosquito repellency and waning for housing modification.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per housing modification (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'ScreeningHousingModification', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for housing modification. Killing is conditional on NOT blocking the mosquito before feeding.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'ScreeningHousingModification', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'ScreeningHousingModification')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='ScreeningHousingModification', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(ScreeningHousingModification, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class SimpleBednet(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'Configures the rate of blocking for indoor mosquito feeds on individuals with an ITN; decays over time. ', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 3.75, 'description': 'Unit cost per bednet (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'SimpleBednet', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of the rate at which mosquitoes die, conditional on a successfully blocked feed; decays over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['MALARIA_SIM', 'VECTOR_SIM'], 'Usage_Config': {'description': 'The user-defined WaningEffects to determine when and if an individual is using a bed net. ', 'type': 'idmType:WaningEffect'}, 'class': 'SimpleBednet', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleBednet')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=3.75, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='SimpleBednet', Killing_Config=None, New_Property_Value=None, Sim_Types=['MALARIA_SIM', 'VECTOR_SIM'], Usage_Config=None, iv_type='IndividualTargeted', **kwargs):
        super(SimpleBednet, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Usage_Config = Usage_Config
        self.iv_type = iv_type



class SimpleBoosterVaccine(BaseCampaign):
    _definition = {'Boost_Effect': {'default': 1, 'description': 'Specifies the boosting effect on [acquisition/transmission/mortality] immunity for previously exposed individuals (either natural or vaccine-derived).', 'max': 1, 'min': 0, 'type': 'float'}, 'Boost_Threshold': {'default': 0, 'description': 'Specifies how much immunity is required before the vaccine changes from a priming effect to a boosting effect.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Efficacy_Is_Multiplicative': {'default': 1, 'description': 'The overall vaccine efficacy when individuals receive more than one vaccine. When set to true (1), the vaccine efficacies are multiplied together; when set to false (0), the efficacies are additive.', 'type': 'bool'}, 'Intervention_Name': {'default': 'SimpleBoosterVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Prime_Effect': {'default': 1, 'description': 'Specifies the priming effect on [acquisition/transmission/mortality] immunity for naive individuals (without natural or vaccine-derived immunity).', 'max': 1, 'min': 0, 'type': 'float'}, 'Sim_Types': ['*'], 'Vaccine_Take': {'default': 1, 'description': 'The rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.', 'max': 1, 'min': 0, 'type': 'float'}, 'Vaccine_Type': {'default': 'Generic', 'description': 'The type of vaccine to distribute in a vaccine intervention.', 'enum': ['Generic', 'TransmissionBlocking', 'AcquisitionBlocking', 'MortalityBlocking'], 'type': 'enum'}, 'Waning_Config': {'description': 'The configuration of Ivermectin killing efficacy and waning over time.', 'type': 'idmType:WaningEffect'}, 'class': 'SimpleBoosterVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleBoosterVaccine')

    def __init__(self, Boost_Effect=1, Boost_Threshold=0, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Efficacy_Is_Multiplicative=True, Intervention_Name='SimpleBoosterVaccine', New_Property_Value=None, Prime_Effect=1, Sim_Types=['*'], Vaccine_Take=1, Vaccine_Type=SimpleBoosterVaccine_Vaccine_Type_Enum.Generic, Waning_Config=[], iv_type='IndividualTargeted', **kwargs):
        super(SimpleBoosterVaccine, self).__init__(**kwargs)
        self.Boost_Effect = Boost_Effect
        self.Boost_Threshold = Boost_Threshold
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Efficacy_Is_Multiplicative = Efficacy_Is_Multiplicative
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Prime_Effect = Prime_Effect
        self.Sim_Types = Sim_Types
        self.Vaccine_Take = Vaccine_Take
        self.Vaccine_Type = Vaccine_Type.name if isinstance(Vaccine_Type, Enum) else Vaccine_Type
        self.Waning_Config = Waning_Config
        self.iv_type = iv_type



class SimpleDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'SimpleDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['*'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'SimpleDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=SimpleDiagnostic_Event_Or_Config_Enum.Config, Intervention_Name='SimpleDiagnostic', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['*'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(SimpleDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class SimpleHealthSeekingBehavior(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The configuration of an actual intervention sought. Selects a class for the intervention and configures the parameters specific for that intervention class.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Actual_IndividualIntervention_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event of an actual intervention sought. Selects a class for the intervention and configures the parameters specific for that intervention class. See the list of available events for possible values.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'SimpleHealthSeekingBehavior', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'Single_Use': {'default': 1, 'description': 'If set to true (1), the health-seeking behavior gets used once and discarded. If set to false (0), it remains indefinitely.', 'type': 'bool'}, 'Tendency': {'default': 1, 'description': 'The probability of seeking healthcare.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'SimpleHealthSeekingBehavior', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleHealthSeekingBehavior')

    def __init__(self, Actual_IndividualIntervention_Config=None, Actual_IndividualIntervention_Event='NoTrigger', Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=SimpleHealthSeekingBehavior_Event_Or_Config_Enum.Config, Intervention_Name='SimpleHealthSeekingBehavior', New_Property_Value=None, Sim_Types=['*'], Single_Use=True, Tendency=1, iv_type='IndividualTargeted', **kwargs):
        super(SimpleHealthSeekingBehavior, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Config = Actual_IndividualIntervention_Config
        self.Actual_IndividualIntervention_Event = Actual_IndividualIntervention_Event
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Single_Use = Single_Use
        self.Tendency = Tendency
        self.iv_type = iv_type



class SimpleHousingModification(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of pre-feed mosquito repellency and waning for housing modification.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per housing modification (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'SimpleHousingModification', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for housing modification. Killing is conditional on NOT blocking the mosquito before feeding.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'SimpleHousingModification', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleHousingModification')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='SimpleHousingModification', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(SimpleHousingModification, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class SimpleIndividualRepellent(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of efficacy and waning for individual repellent.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per repellent (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'SimpleIndividualRepellent', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'SimpleIndividualRepellent', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleIndividualRepellent')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='SimpleIndividualRepellent', New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(SimpleIndividualRepellent, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class SimpleVaccine(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Efficacy_Is_Multiplicative': {'default': 1, 'description': 'The overall vaccine efficacy when individuals receive more than one vaccine. When set to true (1), the vaccine efficacies are multiplied together; when set to false (0), the efficacies are additive.', 'type': 'bool'}, 'Intervention_Name': {'default': 'SimpleVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'Vaccine_Take': {'default': 1, 'description': 'The rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.', 'max': 1, 'min': 0, 'type': 'float'}, 'Vaccine_Type': {'default': 'Generic', 'description': 'The type of vaccine to distribute in a vaccine intervention.', 'enum': ['Generic', 'TransmissionBlocking', 'AcquisitionBlocking', 'MortalityBlocking'], 'type': 'enum'}, 'Waning_Config': {'description': 'The configuration of Ivermectin killing efficacy and waning over time.', 'type': 'idmType:WaningEffect'}, 'class': 'SimpleVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SimpleVaccine')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Efficacy_Is_Multiplicative=True, Intervention_Name='SimpleVaccine', New_Property_Value=None, Sim_Types=['*'], Vaccine_Take=1, Vaccine_Type=SimpleVaccine_Vaccine_Type_Enum.Generic, Waning_Config=[], iv_type='IndividualTargeted', **kwargs):
        super(SimpleVaccine, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Efficacy_Is_Multiplicative = Efficacy_Is_Multiplicative
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Vaccine_Take = Vaccine_Take
        self.Vaccine_Type = Vaccine_Type.name if isinstance(Vaccine_Type, Enum) else Vaccine_Type
        self.Waning_Config = Waning_Config
        self.iv_type = iv_type



class SmearDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'SmearDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': [], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'SmearDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SmearDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=SmearDiagnostic_Event_Or_Config_Enum.Config, Intervention_Name='SmearDiagnostic', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=[], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(SmearDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class SpatialRepellentHousingModification(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of pre-feed mosquito repellency and waning for housing modification.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 8, 'description': 'Unit cost per housing modification (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'SpatialRepellentHousingModification', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for housing modification. Killing is conditional on NOT blocking the mosquito before feeding.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'SpatialRepellentHousingModification', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'SpatialRepellentHousingModification')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=8, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Intervention_Name='SpatialRepellentHousingModification', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='IndividualTargeted', **kwargs):
        super(SpatialRepellentHousingModification, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class StiCoInfectionDiagnostic(BaseCampaign):
    _definition = {'Base_Sensitivity': {'default': 1, 'description': 'The sensitivity of the diagnostic. This sets the proportion of the time that individuals with the condition being tested receive a positive diagnostic test. When set to zero, then individuals who have the condition always receive a false-negative diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Base_Specificity': {'default': 1, 'description': 'The specificity of the diagnostic. This sets the proportion of the time that individuals without the condition being tested receive a negative diagnostic test. When set to 1, the diagnostic always accurately reflects the lack of having the condition. When set to zero, then individuals who do not have the condition always receive a false-positive diagnostic test.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': "The unit 'cost' assigned to the diagnostic. Setting Cost_To_Consumer to zero for all other interventions, and to a non-zero amount for one intervention, provides a convenient way to track the number of times the intervention has been applied in a simulation.", 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Days_To_Diagnosis': {'default': 0, 'description': 'The number of days from test until diagnosis.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Event_Or_Config': {'default': 'Config', 'description': 'Specifies whether the current intervention (or a positive diagnosis, depending on the intervention class) distributes a nested intervention (the Config option) or an event will be broadcast which may trigger other interventions in the campaign file (the Event option). ', 'enum': ['Config', 'Event'], 'type': 'enum'}, 'Intervention_Name': {'default': 'StiCoInfectionDiagnostic', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Positive_Diagnosis_Config': {'depends-on': {'Event_Or_Config': 'Config'}, 'description': 'The intervention distributed to individuals if they test positive. Only used when Event_Or_Config is set to Config.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Positive_Diagnosis_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'If the test is positive, this specifies an event that can trigger another intervention when the event occurs. Only used if Event_Or_Config is set to Event.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['STI_SIM', 'HIV_SIM'], 'Treatment_Fraction': {'default': 1, 'description': 'The fraction of positive diagnoses that are treated.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'StiCoInfectionDiagnostic', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'StiCoInfectionDiagnostic')

    def __init__(self, Base_Sensitivity=1, Base_Specificity=1, Cost_To_Consumer=1, Days_To_Diagnosis=0, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Event_Or_Config=StiCoInfectionDiagnostic_Event_Or_Config_Enum.Config, Intervention_Name='StiCoInfectionDiagnostic', New_Property_Value=None, Positive_Diagnosis_Config=None, Positive_Diagnosis_Event='NoTrigger', Sim_Types=['STI_SIM', 'HIV_SIM'], Treatment_Fraction=1, iv_type='IndividualTargeted', **kwargs):
        super(StiCoInfectionDiagnostic, self).__init__(**kwargs)
        self.Base_Sensitivity = Base_Sensitivity
        self.Base_Specificity = Base_Specificity
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Days_To_Diagnosis = Days_To_Diagnosis
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Event_Or_Config = Event_Or_Config.name if isinstance(Event_Or_Config, Enum) else Event_Or_Config
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Positive_Diagnosis_Config = Positive_Diagnosis_Config
        self.Positive_Diagnosis_Event = Positive_Diagnosis_Event
        self.Sim_Types = Sim_Types
        self.Treatment_Fraction = Treatment_Fraction
        self.iv_type = iv_type



class TBHIVConfigurableTBdrug(BaseCampaign):
    _definition = {'Active_Multiplier': {'default': 1, 'description': 'Multiplier of clearance/inactivation if active TB on drug treatment.', 'max': 1, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 1, 'description': 'Unit cost per drug (unamortized).', 'max': 99999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Dose_Interval': {'default': 1, 'description': 'The interval between doses, in days.', 'max': 99999, 'min': 0, 'type': 'float'}, 'Drug_CMax': {'default': 1, 'description': 'The maximum drug concentration that can be used, and is in the same units as Drug_PKPD_C50.', 'max': 10000, 'min': 0, 'type': 'float'}, 'Drug_PKPD_C50': {'default': 1, 'description': 'The concentration at which drug killing rates are half of the maximum. Must use the same units as Drug_Cmax.', 'max': 5000, 'min': 0, 'type': 'float'}, 'Drug_Vd': {'default': 1, 'description': 'The volume of drug distribution. This value is the ratio of the volume of the second compartment to the volume of the first compartment in a two-compartment model, and is dimensionless.', 'max': 10000, 'min': 0, 'type': 'float'}, 'Durability_Profile': {'default': 'FIXED_DURATION_CONSTANT_EFFECT', 'description': 'The profile of durability decay.', 'enum': ['FIXED_DURATION_CONSTANT_EFFECT', 'CONCENTRATION_VERSUS_TIME'], 'type': 'enum'}, 'Fraction_Defaulters': {'default': 0, 'description': 'The fraction of individuals who will not finish their drug treatment.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'TBHIVConfigurableTBdrug', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Latency_Multiplier': {'default': 1, 'description': 'Multiplier of clearance/inactivation if latent TB on drug treatment.', 'max': 1, 'min': 0, 'type': 'float'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Primary_Decay_Time_Constant': {'default': 1, 'description': 'The primary decay time constant (in days) of the decay profile.', 'max': 999999, 'min': 0, 'type': 'float'}, 'Remaining_Doses': {'default': 0, 'description': 'The remaining doses in an intervention; enter a negative number for unlimited doses.', 'max': 999999, 'min': -1, 'type': 'integer'}, 'Secondary_Decay_Time_Constant': {'default': 1, 'description': 'The secondary decay time constant of the durability profile. ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Sim_Types': ['TBHIV_SIM'], 'TB_Drug_Name': {'default': 'UNINITIALIZED STRING', 'description': 'No Description Yet', 'type': 'string'}, 'class': 'TBHIVConfigurableTBdrug', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'TBHIVConfigurableTBdrug')

    def __init__(self, Active_Multiplier=1, Cost_To_Consumer=1, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Dose_Interval=1, Drug_CMax=1, Drug_PKPD_C50=1, Drug_Vd=1, Durability_Profile=TBHIVConfigurableTBdrug_Durability_Profile_Enum.FIXED_DURATION_CONSTANT_EFFECT, Fraction_Defaulters=0, Intervention_Name='TBHIVConfigurableTBdrug', Latency_Multiplier=1, New_Property_Value=None, Primary_Decay_Time_Constant=1, Remaining_Doses=0, Secondary_Decay_Time_Constant=1, Sim_Types=['TBHIV_SIM'], TB_Drug_Name='UNINITIALIZED STRING', iv_type='IndividualTargeted', **kwargs):
        super(TBHIVConfigurableTBdrug, self).__init__(**kwargs)
        self.Active_Multiplier = Active_Multiplier
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Dose_Interval = Dose_Interval
        self.Drug_CMax = Drug_CMax
        self.Drug_PKPD_C50 = Drug_PKPD_C50
        self.Drug_Vd = Drug_Vd
        self.Durability_Profile = Durability_Profile.name if isinstance(Durability_Profile, Enum) else Durability_Profile
        self.Fraction_Defaulters = Fraction_Defaulters
        self.Intervention_Name = Intervention_Name
        self.Latency_Multiplier = Latency_Multiplier
        self.New_Property_Value = New_Property_Value
        self.Primary_Decay_Time_Constant = Primary_Decay_Time_Constant
        self.Remaining_Doses = Remaining_Doses
        self.Secondary_Decay_Time_Constant = Secondary_Decay_Time_Constant
        self.Sim_Types = Sim_Types
        self.TB_Drug_Name = TB_Drug_Name
        self.iv_type = iv_type



class UsageDependentBednet(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'Configures the rate of blocking for indoor mosquito feeds on individuals with an ITN; decays over time. ', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 3.75, 'description': 'Unit cost per bednet (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Discard_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event that is broadcast when an individual discards their bed net, either by replacing an existing net or due to the expiration timer. ', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Expiration_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The type of distribution to use when calculating the time to discard a bed net.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Expiration_Percentage_Period_1': {'default': 0.5, 'depends-on': {'Expiration_Distribution_Type': 'DUAL_TIMESCALE_DURATION'}, 'description': 'The percentage of draws where the first time-scale is used. ', 'max': 1, 'min': 0, 'type': 'float'}, 'Expiration_Period': {'default': 6, 'depends-on': {'Expiration_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'The distribution period for when a bed net expires. ', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Expiration_Period_1': {'default': 6, 'depends-on': {'Expiration_Distribution_Type': 'DUAL_TIMESCALE_DURATION'}, 'description': 'The decay length of the first time-scale.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Expiration_Period_2': {'default': 6, 'depends-on': {'Expiration_Distribution_Type': 'DUAL_TIMESCALE_DURATION'}, 'description': 'The decay length of the second time-scale.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Expiration_Period_Max': {'default': 1, 'depends-on': {'Expiration_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum duration of use for a bed net.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Expiration_Period_Mean': {'default': 6, 'depends-on': {'Expiration_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The mean of the duration of use for a bed net.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Expiration_Period_Min': {'default': 0, 'depends-on': {'Expiration_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum duration of use for a bed net.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Expiration_Period_Std_Dev': {'default': 1, 'depends-on': {'Expiration_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation for the duration of use for a bed net.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'UsageDependentBednet', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of the rate at which mosquitoes die, conditional on a successfully blocked feed; decays over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Received_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'This parameter broadcasts when a new net is received, either the first net or a replacement net.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Sim_Types': ['MALARIA_SIM', 'VECTOR_SIM'], 'Usage_Config_List': {'description': 'The list of WaningEffects whose effects are multiplied together to get the usage effect.', 'type': 'list', 'item_type': {'type': 'object'}, 'default': []}, 'Using_Event': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'This parameter broadcasts each time step in which a bed net is used.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'class': 'UsageDependentBednet', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'UsageDependentBednet')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=3.75, Discard_Event='NoTrigger', Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Expiration_Distribution_Type=UsageDependentBednet_Expiration_Distribution_Type_Enum.NOT_INITIALIZED, Expiration_Percentage_Period_1=0.5, Expiration_Period=6, Expiration_Period_1=6, Expiration_Period_2=6, Expiration_Period_Max=1, Expiration_Period_Mean=6, Expiration_Period_Min=0, Expiration_Period_Std_Dev=1, Intervention_Name='UsageDependentBednet', Killing_Config=None, New_Property_Value=None, Received_Event='NoTrigger', Sim_Types=['MALARIA_SIM', 'VECTOR_SIM'], Usage_Config_List=[], Using_Event='NoTrigger', iv_type='IndividualTargeted', **kwargs):
        super(UsageDependentBednet, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Discard_Event = Discard_Event
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Expiration_Distribution_Type = Expiration_Distribution_Type.name if isinstance(Expiration_Distribution_Type, Enum) else Expiration_Distribution_Type
        self.Expiration_Percentage_Period_1 = Expiration_Percentage_Period_1
        self.Expiration_Period = Expiration_Period
        self.Expiration_Period_1 = Expiration_Period_1
        self.Expiration_Period_2 = Expiration_Period_2
        self.Expiration_Period_Max = Expiration_Period_Max
        self.Expiration_Period_Mean = Expiration_Period_Mean
        self.Expiration_Period_Min = Expiration_Period_Min
        self.Expiration_Period_Std_Dev = Expiration_Period_Std_Dev
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Received_Event = Received_Event
        self.Sim_Types = Sim_Types
        self.Usage_Config_List = Usage_Config_List
        self.Using_Event = Using_Event
        self.iv_type = iv_type



class AnimalFeedKill(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'AnimalFeedKill', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy of the targeted stage. Use a waning effect class to specify how this effect decays over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'AnimalFeedKill', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'AnimalFeedKill')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Intervention_Name='AnimalFeedKill', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(AnimalFeedKill, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class ArtificialDiet(BaseCampaign):
    _definition = {'Artificial_Diet_Target': {'default': 'AD_WithinVillage', 'description': 'The targeted deployment of artificial diet.', 'enum': ['AD_WithinVillage', 'AD_OutsideVillage'], 'type': 'enum'}, 'Attraction_Config': {'description': 'The fraction of vector feeds attracted to the artificial diet.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'ArtificialDiet', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'ArtificialDiet', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'ArtificialDiet')

    def __init__(self, Artificial_Diet_Target=ArtificialDiet_Artificial_Diet_Target_Enum.AD_WithinVillage, Attraction_Config=None, Cost_To_Consumer=10, Disqualifying_Properties=[], Intervention_Name='ArtificialDiet', New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(ArtificialDiet, self).__init__(**kwargs)
        self.Artificial_Diet_Target = Artificial_Diet_Target.name if isinstance(Artificial_Diet_Target, Enum) else Artificial_Diet_Target
        self.Attraction_Config = Attraction_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class BirthTriggeredIV(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Config': {'description': 'The configuration of an actual individual intervention sought.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Duration': {'default': -1, 'description': 'The number of days to continue this intervention. ', 'max': 3.40282e+38, 'min': -1, 'type': 'float'}, 'Intervention_Name': {'default': 'BirthTriggeredIV', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Sim_Types': ['*'], 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'class': 'BirthTriggeredIV', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'BirthTriggeredIV')

    def __init__(self, Actual_IndividualIntervention_Config=None, Demographic_Coverage=1, Disqualifying_Properties=[], Duration=-1, Intervention_Name='BirthTriggeredIV', New_Property_Value=None, Property_Restrictions=[], Property_Restrictions_Within_Node=[], Sim_Types=['*'], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=BirthTriggeredIV_Target_Demographic_Enum.Everyone, Target_Gender=BirthTriggeredIV_Target_Gender_Enum.All, Target_Residents_Only=False, iv_type='NodeTargeted', **kwargs):
        super(BirthTriggeredIV, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Config = Actual_IndividualIntervention_Config
        self.Demographic_Coverage = Demographic_Coverage
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Duration = Duration
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Sim_Types = Sim_Types
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.iv_type = iv_type



class ImportPressure(BaseCampaign):
    _definition = {'Antigen': {'default': 0, 'description': 'The antigenic ID of the outbreak infection.', 'max': 10, 'min': 0, 'type': 'integer'}, 'Daily_Import_Pressures': {'ascending': 0, 'default': 0, 'description': 'The rate of per-day importation for each node that the intervention is distributed to.', 'max': 3.40282e+38, 'min': 0, 'type': 'Vector Float'}, 'Durations': {'default': 1, 'description': 'The durations over which to apply import pressure.', 'max': 2147480000.0, 'min': 0, 'type': 'Vector Int'}, 'Genome': {'default': 0, 'description': 'The genetic ID of the outbreak infection.', 'max': 16777200.0, 'min': -1, 'type': 'integer'}, 'Import_Age': {'default': 365, 'description': 'The age (in days) of infected import cases.', 'max': 43800, 'min': 0, 'type': 'float'}, 'Incubation_Period_Override': {'default': -1, 'description': 'The incubation period, in days, that infected individuals will go through before becoming infectious. This value overrides the incubation period set in the configuration file. Set to -1 to honor the configuration parameter settings.', 'max': 2147480000.0, 'min': -1, 'type': 'integer'}, 'Number_Cases_Per_Node': {'default': 1, 'description': 'The number of new cases of Outbreak to import (per node). ', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'Sim_Types': ['GENERIC_SIM'], 'class': 'ImportPressure', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'ImportPressure')

    def __init__(self, Antigen=0, Daily_Import_Pressures=0, Durations=1, Genome=0, Import_Age=365, Incubation_Period_Override=-1, Number_Cases_Per_Node=1, Sim_Types=['GENERIC_SIM'], iv_type='NodeTargeted', **kwargs):
        super(ImportPressure, self).__init__(**kwargs)
        self.Antigen = Antigen
        self.Daily_Import_Pressures = Daily_Import_Pressures
        self.Durations = Durations
        self.Genome = Genome
        self.Import_Age = Import_Age
        self.Incubation_Period_Override = Incubation_Period_Override
        self.Number_Cases_Per_Node = Number_Cases_Per_Node
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class InputEIR(BaseCampaign):
    _definition = {'Age_Dependence': {'default': 'OFF', 'description': 'Determines how InputEIR depends on the age of the target.', 'enum': ['OFF', 'LINEAR', 'SURFACE_AREA_DEPENDENT'], 'type': 'enum'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'InputEIR', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Monthly_EIR': {'description': 'An array of 12 elements that contain an entomological inoculation rate (EIR) for each month. Each value should be between 0 and 1000.', 'type': 'list', 'item_type': {'description': 'An array of 12 elements that contain an entomological inoculation rate (EIR) for each month. Each value should be between 0 and 1000.', 'max': 1000, 'min': 0, 'type': 'float'}, 'default': []}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['MALARIA_SIM'], 'class': 'InputEIR', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'InputEIR')

    def __init__(self, Age_Dependence=InputEIR_Age_Dependence_Enum.OFF, Disqualifying_Properties=[], Intervention_Name='InputEIR', Monthly_EIR=[], New_Property_Value=None, Sim_Types=['MALARIA_SIM'], iv_type='NodeTargeted', **kwargs):
        super(InputEIR, self).__init__(**kwargs)
        self.Age_Dependence = Age_Dependence.name if isinstance(Age_Dependence, Enum) else Age_Dependence
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Monthly_EIR = Monthly_EIR
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class InsectKillingFence(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'InsectKillingFence', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration for the effects of killing of the targeted stage. Use a waning effect class to specify how this effect decays over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'InsectKillingFence', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'InsectKillingFence')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Intervention_Name='InsectKillingFence', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(InsectKillingFence, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class Larvicides(BaseCampaign):
    _definition = {'Blocking_Config': {'description': 'The configuration of larval habitat reduction and waning for targeted stage.', 'type': 'idmType:WaningEffect'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Habitat_Target': {'default': 'TEMPORARY_RAINFALL', 'description': 'The larval habitat type targeted for larvicide.', 'enum': ['TEMPORARY_RAINFALL', 'WATER_VEGETATION', 'HUMAN_POPULATION', 'CONSTANT', 'BRACKISH_SWAMP', 'MARSHY_STREAM', 'LINEAR_SPLINE', 'ALL_HABITATS'], 'type': 'enum'}, 'Intervention_Name': {'default': 'Larvicides', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of larval killing efficacy and waning for targeted stage.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'Larvicides', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'Larvicides')

    def __init__(self, Blocking_Config=None, Cost_To_Consumer=10, Disqualifying_Properties=[], Habitat_Target=Larvicides_Habitat_Target_Enum.TEMPORARY_RAINFALL, Intervention_Name='Larvicides', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(Larvicides, self).__init__(**kwargs)
        self.Blocking_Config = Blocking_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Habitat_Target = Habitat_Target.name if isinstance(Habitat_Target, Enum) else Habitat_Target
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class MalariaChallenge(BaseCampaign):
    _definition = {'Challenge_Type': {'default': 'InfectiousBites', 'description': 'The type of malaria challenge.', 'enum': ['InfectiousBites', 'Sporozoites'], 'type': 'enum'}, 'Coverage': {'default': 1, 'description': 'The fraction of individuals receiving an intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Infectious_Bite_Count': {'default': 1, 'description': 'The number of infectious bites. ', 'max': 1000, 'min': 0, 'type': 'integer'}, 'Intervention_Name': {'default': 'MalariaChallenge', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['MALARIA_SIM'], 'Sporozoite_Count': {'default': 1, 'description': 'The number of sporozoites.', 'max': 1000, 'min': 0, 'type': 'integer'}, 'class': 'MalariaChallenge', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'MalariaChallenge')

    def __init__(self, Challenge_Type=MalariaChallenge_Challenge_Type_Enum.InfectiousBites, Coverage=1, Disqualifying_Properties=[], Infectious_Bite_Count=1, Intervention_Name='MalariaChallenge', New_Property_Value=None, Sim_Types=['MALARIA_SIM'], Sporozoite_Count=1, iv_type='NodeTargeted', **kwargs):
        super(MalariaChallenge, self).__init__(**kwargs)
        self.Challenge_Type = Challenge_Type.name if isinstance(Challenge_Type, Enum) else Challenge_Type
        self.Coverage = Coverage
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Infectious_Bite_Count = Infectious_Bite_Count
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Sporozoite_Count = Sporozoite_Count
        self.iv_type = iv_type



class MigrateFamily(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Duration_At_Node_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The shape of the distribution for the amount of time spent at the destination node for intervention-based migration.', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Duration_At_Node_Exponential_Period': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'The period (1/rate) to use for an exponential distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to EXPONENTIAL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Fixed': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'FIXED_DURATION'}, 'description': 'The value to use for a fixed distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to FIXED_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Gausian_Mean': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The mean value to use for a Gaussian distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Gausian_StdDev': {'default': 1, 'depends-on': {'Duration_At_Node_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation to use for a Gaussian distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Poisson_Mean': {'default': 6, 'depends-on': {'Duration_At_Node_Distribution_Type': 'POISSON_DURATION'}, 'description': 'The mean to use for a Poisson distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to POISSON_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Uniform_Max': {'default': 1, 'depends-on': {'Duration_At_Node_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum value to use for a uniform distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_At_Node_Uniform_Min': {'default': 0, 'depends-on': {'Duration_At_Node_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum value to use for a uniform distribution for the amount of time spent at the destination node when Duration_At_Node_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Distribution_Type': {'default': 'NOT_INITIALIZED', 'description': 'The shape of the distribution of the number of days to wait before migrating to the destination node for intervention-based migration. ', 'enum': ['NOT_INITIALIZED', 'FIXED_DURATION', 'UNIFORM_DURATION', 'GAUSSIAN_DURATION', 'EXPONENTIAL_DURATION', 'POISSON_DURATION', 'LOG_NORMAL_DURATION', 'BIMODAL_DURATION', 'PIECEWISE_CONSTANT', 'PIECEWISE_LINEAR', 'WEIBULL_DURATION', 'DUAL_TIMESCALE_DURATION'], 'type': 'enum'}, 'Duration_Before_Leaving_Exponential_Period': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'EXPONENTIAL_DURATION'}, 'description': 'The period (1/rate) to use for an exponential distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to EXPONENTIAL_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Fixed': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'FIXED_DURATION'}, 'description': 'The value to use for a fixed distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to FIXED_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Gausian_Mean': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The mean value to use for a Gaussian distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Gausian_StdDev': {'default': 1, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'GAUSSIAN_DURATION'}, 'description': 'The standard deviation to use for a Gaussian distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to GAUSSIAN_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Poisson_Mean': {'default': 6, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'POISSON_DURATION'}, 'description': 'The mean to use for a Poisson distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to POISSON_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Uniform_Max': {'default': 1, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The maximum value to use for a uniform distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Duration_Before_Leaving_Uniform_Min': {'default': 0, 'depends-on': {'Duration_Before_Leaving_Distribution_Type': 'UNIFORM_DURATION'}, 'description': 'The minimum value to use for a uniform distribution of the number of days to wait before migrating to the destination node when Duration_Before_Leaving_Distribution_Type is set to UNIFORM_DURATION.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'MigrateFamily', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Is_Moving': {'default': 0, 'description': 'Set to true (1) to indicate the individual is permanently moving to a new home node for intervention-based migration.', 'type': 'bool'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'NodeID_To_Migrate_To': {'default': 0, 'description': 'The destination node ID for intervention-based migration.', 'max': 4294970000.0, 'min': 0, 'type': 'integer'}, 'Sim_Types': ['*'], 'class': 'MigrateFamily', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'MigrateFamily')

    def __init__(self, Disqualifying_Properties=[], Duration_At_Node_Distribution_Type=MigrateFamily_Duration_At_Node_Distribution_Type_Enum.NOT_INITIALIZED, Duration_At_Node_Exponential_Period=6, Duration_At_Node_Fixed=6, Duration_At_Node_Gausian_Mean=6, Duration_At_Node_Gausian_StdDev=1, Duration_At_Node_Poisson_Mean=6, Duration_At_Node_Uniform_Max=1, Duration_At_Node_Uniform_Min=0, Duration_Before_Leaving_Distribution_Type=MigrateFamily_Duration_Before_Leaving_Distribution_Type_Enum.NOT_INITIALIZED, Duration_Before_Leaving_Exponential_Period=6, Duration_Before_Leaving_Fixed=6, Duration_Before_Leaving_Gausian_Mean=6, Duration_Before_Leaving_Gausian_StdDev=1, Duration_Before_Leaving_Poisson_Mean=6, Duration_Before_Leaving_Uniform_Max=1, Duration_Before_Leaving_Uniform_Min=0, Intervention_Name='MigrateFamily', Is_Moving=False, New_Property_Value=None, NodeID_To_Migrate_To=0, Sim_Types=['*'], iv_type='NodeTargeted', **kwargs):
        super(MigrateFamily, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Duration_At_Node_Distribution_Type = Duration_At_Node_Distribution_Type.name if isinstance(Duration_At_Node_Distribution_Type, Enum) else Duration_At_Node_Distribution_Type
        self.Duration_At_Node_Exponential_Period = Duration_At_Node_Exponential_Period
        self.Duration_At_Node_Fixed = Duration_At_Node_Fixed
        self.Duration_At_Node_Gausian_Mean = Duration_At_Node_Gausian_Mean
        self.Duration_At_Node_Gausian_StdDev = Duration_At_Node_Gausian_StdDev
        self.Duration_At_Node_Poisson_Mean = Duration_At_Node_Poisson_Mean
        self.Duration_At_Node_Uniform_Max = Duration_At_Node_Uniform_Max
        self.Duration_At_Node_Uniform_Min = Duration_At_Node_Uniform_Min
        self.Duration_Before_Leaving_Distribution_Type = Duration_Before_Leaving_Distribution_Type.name if isinstance(Duration_Before_Leaving_Distribution_Type, Enum) else Duration_Before_Leaving_Distribution_Type
        self.Duration_Before_Leaving_Exponential_Period = Duration_Before_Leaving_Exponential_Period
        self.Duration_Before_Leaving_Fixed = Duration_Before_Leaving_Fixed
        self.Duration_Before_Leaving_Gausian_Mean = Duration_Before_Leaving_Gausian_Mean
        self.Duration_Before_Leaving_Gausian_StdDev = Duration_Before_Leaving_Gausian_StdDev
        self.Duration_Before_Leaving_Poisson_Mean = Duration_Before_Leaving_Poisson_Mean
        self.Duration_Before_Leaving_Uniform_Max = Duration_Before_Leaving_Uniform_Max
        self.Duration_Before_Leaving_Uniform_Min = Duration_Before_Leaving_Uniform_Min
        self.Intervention_Name = Intervention_Name
        self.Is_Moving = Is_Moving
        self.New_Property_Value = New_Property_Value
        self.NodeID_To_Migrate_To = NodeID_To_Migrate_To
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class MosquitoRelease(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 0, 'description': 'Cost of each mosquito release.', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'MosquitoRelease', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Mated_Genetics': {'depends-on': {'Released_Gender': 'VECTOR_FEMALE'}, 'description': 'The genetic properties of the mate if released mosquitoes have mated, e.g. HEG and pesticide resistance.', 'HEG': {'default': 'WILD', 'description': 'HEG characteristics of released mosquitoes.', 'enum': ['WILD', 'HALF', 'FULL', 'NotMated'], 'type': 'enum'}, 'Pesticide_Resistance': {'default': 'WILD', 'description': 'The pesticide resistance characteristics of released mosquitoes.', 'enum': ['WILD', 'HALF', 'FULL', 'NotMated'], 'type': 'enum'}}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Released_Gender': {'default': 'VECTOR_FEMALE', 'description': 'The gender of released mosquitoes.', 'enum': ['VECTOR_FEMALE', 'VECTOR_MALE'], 'type': 'enum'}, 'Released_Genetics': {'description': 'The genetic properties of the released mosquito, e.g. HEG and pesticide resistance.', 'HEG': {'default': 'WILD', 'description': 'HEG characteristics of released mosquitoes.', 'enum': ['WILD', 'HALF', 'FULL', 'NotMated'], 'type': 'enum'}, 'Pesticide_Resistance': {'default': 'WILD', 'description': 'The pesticide resistance characteristics of released mosquitoes.', 'enum': ['WILD', 'HALF', 'FULL', 'NotMated'], 'type': 'enum'}}, 'Released_Number': {'default': 10000, 'description': 'The number of mosquitoes released in the intervention.', 'max': 100000000.0, 'min': 1, 'type': 'integer'}, 'Released_Species': {'default': 'UNINITIALIZED STRING', 'description': 'The name of the released mosquito species.', 'type': 'Constrained String', 'value_source': '<configuration>:Vector_Species_Params.*'}, 'Released_Sterility': {'default': 'VECTOR_FERTILE', 'description': 'The sterility of released mosquitoes.', 'enum': ['VECTOR_FERTILE', 'VECTOR_STERILE'], 'type': 'enum'}, 'Released_Wolbachia': {'default': 'WOLBACHIA_FREE', 'description': 'The Wolbachia type of released mosquitoes.', 'enum': ['WOLBACHIA_FREE', 'VECTOR_WOLBACHIA_A', 'VECTOR_WOLBACHIA_B', 'VECTOR_WOLBACHIA_AB'], 'type': 'enum'}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM'], 'class': 'MosquitoRelease', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'MosquitoRelease')

    def __init__(self, Cost_To_Consumer=0, Disqualifying_Properties=[], Intervention_Name='MosquitoRelease', Mated_Genetics=None, New_Property_Value=None, Released_Gender=MosquitoRelease_Released_Gender_Enum.VECTOR_FEMALE, Released_Genetics=None, Released_Number=10000, Released_Species='UNINITIALIZED STRING', Released_Sterility=MosquitoRelease_Released_Sterility_Enum.VECTOR_FERTILE, Released_Wolbachia=MosquitoRelease_Released_Wolbachia_Enum.WOLBACHIA_FREE, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM'], iv_type='NodeTargeted', **kwargs):
        super(MosquitoRelease, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Mated_Genetics = Mated_Genetics
        self.New_Property_Value = New_Property_Value
        self.Released_Gender = Released_Gender.name if isinstance(Released_Gender, Enum) else Released_Gender
        self.Released_Genetics = Released_Genetics
        self.Released_Number = Released_Number
        self.Released_Species = Released_Species
        self.Released_Sterility = Released_Sterility.name if isinstance(Released_Sterility, Enum) else Released_Sterility
        self.Released_Wolbachia = Released_Wolbachia.name if isinstance(Released_Wolbachia, Enum) else Released_Wolbachia
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class MultiNodeInterventionDistributor(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'MultiNodeInterventionDistributor', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Node_Intervention_List': {'description': 'A list of nested JSON objects for the multi-node-level interventions to be distributed by this intervention.', 'type': 'idmAbstractType:NodeIntervention'}, 'Sim_Types': ['*'], 'class': 'MultiNodeInterventionDistributor', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'MultiNodeInterventionDistributor')

    def __init__(self, Disqualifying_Properties=[], Intervention_Name='MultiNodeInterventionDistributor', New_Property_Value=None, Node_Intervention_List=None, Sim_Types=['*'], iv_type='NodeTargeted', **kwargs):
        super(MultiNodeInterventionDistributor, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Node_Intervention_List = Node_Intervention_List
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class NodeLevelHealthTriggeredIV(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Config': {'description': 'The configuration of an actual individual intervention sought.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Actual_NodeIntervention_Config': {'description': 'The configuration of the actual node-level intervention sought. ', 'type': 'idmAbstractType:NodeIntervention'}, 'Blackout_Event_Trigger': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event to broadcast if an intervention cannot be distributed due to the Blackout_Period.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Blackout_On_First_Occurrence': {'default': 0, 'description': 'If set to true (1), individuals will enter the blackout period after the first occurrence of an event in the Trigger_Condition_List.', 'type': 'bool'}, 'Blackout_Period': {'default': 0, 'description': 'After the initial intervention distribution, the time, in days, to wait before distributing the intervention again. If it cannot distribute due to the blackout period, it will broadcast the user-defined Blackout_Event_Trigger.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Distribute_On_Return_Home': {'default': 0, 'description': 'When set to true (1), individuals will receive an intervention upon returning home if that intervention was originally distributed while the individual was away.', 'type': 'bool'}, 'Duration': {'default': -1, 'description': 'The number of days to continue this intervention. ', 'max': 3.40282e+38, 'min': -1, 'type': 'float'}, 'Intervention_Name': {'default': 'NodeLevelHealthTriggeredIV', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Sim_Types': ['*'], 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Trigger_Condition_List': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'A list of events that trigger a health seeking intervention.', 'type': 'Vector String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'class': 'NodeLevelHealthTriggeredIV', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'NodeLevelHealthTriggeredIV')

    def __init__(self, Actual_IndividualIntervention_Config=None, Actual_NodeIntervention_Config=None, Blackout_Event_Trigger='NoTrigger', Blackout_On_First_Occurrence=False, Blackout_Period=0, Demographic_Coverage=1, Disqualifying_Properties=[], Distribute_On_Return_Home=False, Duration=-1, Intervention_Name='NodeLevelHealthTriggeredIV', New_Property_Value=None, Node_Property_Restrictions=[], Property_Restrictions=[], Property_Restrictions_Within_Node=[], Sim_Types=['*'], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=NodeLevelHealthTriggeredIV_Target_Demographic_Enum.Everyone, Target_Gender=NodeLevelHealthTriggeredIV_Target_Gender_Enum.All, Target_Residents_Only=False, Trigger_Condition_List='NoTrigger', iv_type='NodeTargeted', **kwargs):
        super(NodeLevelHealthTriggeredIV, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Config = Actual_IndividualIntervention_Config
        self.Actual_NodeIntervention_Config = Actual_NodeIntervention_Config
        self.Blackout_Event_Trigger = Blackout_Event_Trigger
        self.Blackout_On_First_Occurrence = Blackout_On_First_Occurrence
        self.Blackout_Period = Blackout_Period
        self.Demographic_Coverage = Demographic_Coverage
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Distribute_On_Return_Home = Distribute_On_Return_Home
        self.Duration = Duration
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Sim_Types = Sim_Types
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Trigger_Condition_List = Trigger_Condition_List
        self.iv_type = iv_type



class NodeLevelHealthTriggeredIVScaleUpSwitch(BaseCampaign):
    _definition = {'Actual_IndividualIntervention_Config': {'description': 'The configuration of an actual individual intervention sought.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Actual_NodeIntervention_Config': {'description': 'The configuration of the actual node-level intervention sought. ', 'type': 'idmAbstractType:NodeIntervention'}, 'Blackout_Event_Trigger': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'The event to broadcast if an intervention cannot be distributed due to the Blackout_Period.', 'type': 'Constrained String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'Blackout_On_First_Occurrence': {'default': 0, 'description': 'If set to true (1), individuals will enter the blackout period after the first occurrence of an event in the Trigger_Condition_List.', 'type': 'bool'}, 'Blackout_Period': {'default': 0, 'description': 'After the initial intervention distribution, the time, in days, to wait before distributing the intervention again. If it cannot distribute due to the blackout period, it will broadcast the user-defined Blackout_Event_Trigger.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Demographic_Coverage': {'default': 1, 'description': 'The fraction of individuals in the target demographic that will receive this intervention.', 'max': 1, 'min': 0, 'type': 'float'}, 'Demographic_Coverage_Time_Profile': {'default': 'Immediate', 'description': 'Profile of ramp up to demographic coverage.', 'enum': ['Immediate', 'Linear', 'Sigmoid'], 'type': 'enum'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Distribute_On_Return_Home': {'default': 0, 'description': 'When set to true (1), individuals will receive an intervention upon returning home if that intervention was originally distributed while the individual was away.', 'type': 'bool'}, 'Duration': {'default': -1, 'description': 'The number of days to continue this intervention. ', 'max': 3.40282e+38, 'min': -1, 'type': 'float'}, 'Initial_Demographic_Coverage': {'default': 0, 'description': 'The initial level of demographic coverage if using linear scale up.', 'max': 1, 'min': 0, 'type': 'float'}, 'Intervention_Name': {'default': 'NodeLevelHealthTriggeredIVScaleUpSwitch', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Node_Property_Restrictions': {'description': 'A list of the NodeProperty key:value pairs, as defined in the demographics file, that nodes must have to be targeted by the intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::NodeProperties.*.Property', 'description': 'Node Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::NodeProperties.*.Values', 'description': 'Node Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Not_Covered_IndividualIntervention_Configs': {'description': 'The array of interventions that is distributed if an individual qualifies according to all parameters except Demographic_Coverage.', 'type': 'idmAbstractType:IndividualIntervention'}, 'Primary_Time_Constant': {'default': 1, 'description': 'The time to full scale-up of demographic coverage.', 'max': 365000, 'min': 0, 'type': 'float'}, 'Property_Restrictions': {'default': [], 'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'Dynamic String Set', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Property_Restrictions_Within_Node': {'description': 'A list of the IndividualProperty key:value pairs, as defined in the demographics file, that individuals must have to be targeted by this intervention.', 'type': 'list', 'item_type': {'type': 'dict', 'key_type': {'constraints': '<demographics>::*.Individual_Properties.*.Property', 'description': 'Individual Property Key from demographics file.', 'type': 'Constrained String'}, 'value_type': {'constraints': '<demographics>::*.Individual_Properties.*.Values', 'description': 'Individual Property Value from demographics file.', 'type': 'String'}}, 'default': []}, 'Sim_Types': ['*'], 'Target_Age_Max': {'default': 3.40282e+38, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The upper end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Age_Min': {'default': 0, 'depends-on': {'Target_Demographic': 'ExplicitAgeRanges,ExplicitAgeRangesAndGender'}, 'description': 'The lower end of ages targeted for an intervention, in years.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Target_Demographic': {'default': 'Everyone', 'description': 'The target demographic group.', 'enum': ['Everyone', 'ExplicitAgeRanges', 'ExplicitAgeRangesAndGender', 'ExplicitGender', 'PossibleMothers', 'ArrivingAirTravellers', 'DepartingAirTravellers', 'ArrivingRoadTravellers', 'DepartingRoadTravellers', 'AllArrivingTravellers', 'AllDepartingTravellers', 'ExplicitDiseaseState'], 'type': 'enum'}, 'Target_Gender': {'default': 'All', 'description': 'Specifies the gender restriction for the intervention.', 'enum': ['All', 'Male', 'Female'], 'type': 'enum'}, 'Target_Residents_Only': {'default': 0, 'description': 'When set to true (1), the intervention is only distributed to individuals that began the simulation in the node (i.e. those that claim the node as their residence).', 'type': 'bool'}, 'Trigger_Condition_List': {'Built-in': ['NoTrigger', 'Births', 'EveryUpdate', 'EveryTimeStep', 'NewInfectionEvent', 'TBActivation', 'NewClinicalCase', 'NewSevereCase', 'DiseaseDeaths', 'OpportunisticInfectionDeath', 'NonDiseaseDeaths', 'TBActivationSmearPos', 'TBActivationSmearNeg', 'TBActivationExtrapulm', 'TBActivationPostRelapse', 'TBPendingRelapse', 'TBActivationPresymptomatic', 'TestPositiveOnSmear', 'ProviderOrdersTBTest', 'TBTestPositive', 'TBTestNegative', 'TBTestDefault', 'TBRestartHSB', 'TBMDRTestPositive', 'TBMDRTestNegative', 'TBMDRTestDefault', 'TBFailedDrugRegimen', 'TBRelapseAfterDrugRegimen', 'TBStartDrugRegimen', 'TBStopDrugRegimen', 'PropertyChange', 'STIDebut', 'StartedART', 'StoppedART', 'InterventionDisqualified', 'HIVNewlyDiagnosed', 'GaveBirth', 'Pregnant', 'Emigrating', 'Immigrating', 'HIVTestedNegative', 'HIVTestedPositive', 'HIVSymptomatic', 'TwelveWeeksPregnant', 'FourteenWeeksPregnant', 'SixWeeksOld', 'EighteenMonthsOld', 'STIPreEmigrating', 'STIPostImmigrating', 'STINewInfection', 'NewExternalHIVInfection', 'NodePropertyChange', 'HappyBirthday', 'EnteredRelationship', 'ExitedRelationship', 'FirstCoitalAct'], 'default': 'NoTrigger', 'description': 'A list of events that trigger a health seeking intervention.', 'type': 'Vector String', 'value_source': "'<configuration>:Listed_Events.*' or Built-in"}, 'class': 'NodeLevelHealthTriggeredIVScaleUpSwitch', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'NodeLevelHealthTriggeredIVScaleUpSwitch')

    def __init__(self, Actual_IndividualIntervention_Config=None, Actual_NodeIntervention_Config=None, Blackout_Event_Trigger='NoTrigger', Blackout_On_First_Occurrence=False, Blackout_Period=0, Demographic_Coverage=1, Demographic_Coverage_Time_Profile=NodeLevelHealthTriggeredIVScaleUpSwitch_Demographic_Coverage_Time_Profile_Enum.Immediate, Disqualifying_Properties=[], Distribute_On_Return_Home=False, Duration=-1, Initial_Demographic_Coverage=0, Intervention_Name='NodeLevelHealthTriggeredIVScaleUpSwitch', New_Property_Value=None, Node_Property_Restrictions=[], Not_Covered_IndividualIntervention_Configs=None, Primary_Time_Constant=1, Property_Restrictions=[], Property_Restrictions_Within_Node=[], Sim_Types=['*'], Target_Age_Max=3.40282e+38, Target_Age_Min=0, Target_Demographic=NodeLevelHealthTriggeredIVScaleUpSwitch_Target_Demographic_Enum.Everyone, Target_Gender=NodeLevelHealthTriggeredIVScaleUpSwitch_Target_Gender_Enum.All, Target_Residents_Only=False, Trigger_Condition_List='NoTrigger', iv_type='NodeTargeted', **kwargs):
        super(NodeLevelHealthTriggeredIVScaleUpSwitch, self).__init__(**kwargs)
        self.Actual_IndividualIntervention_Config = Actual_IndividualIntervention_Config
        self.Actual_NodeIntervention_Config = Actual_NodeIntervention_Config
        self.Blackout_Event_Trigger = Blackout_Event_Trigger
        self.Blackout_On_First_Occurrence = Blackout_On_First_Occurrence
        self.Blackout_Period = Blackout_Period
        self.Demographic_Coverage = Demographic_Coverage
        self.Demographic_Coverage_Time_Profile = Demographic_Coverage_Time_Profile.name if isinstance(Demographic_Coverage_Time_Profile, Enum) else Demographic_Coverage_Time_Profile
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Distribute_On_Return_Home = Distribute_On_Return_Home
        self.Duration = Duration
        self.Initial_Demographic_Coverage = Initial_Demographic_Coverage
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Node_Property_Restrictions = Node_Property_Restrictions
        self.Not_Covered_IndividualIntervention_Configs = Not_Covered_IndividualIntervention_Configs
        self.Primary_Time_Constant = Primary_Time_Constant
        self.Property_Restrictions = Property_Restrictions
        self.Property_Restrictions_Within_Node = Property_Restrictions_Within_Node
        self.Sim_Types = Sim_Types
        self.Target_Age_Max = Target_Age_Max
        self.Target_Age_Min = Target_Age_Min
        self.Target_Demographic = Target_Demographic.name if isinstance(Target_Demographic, Enum) else Target_Demographic
        self.Target_Gender = Target_Gender.name if isinstance(Target_Gender, Enum) else Target_Gender
        self.Target_Residents_Only = Target_Residents_Only
        self.Trigger_Condition_List = Trigger_Condition_List
        self.iv_type = iv_type



class NodePropertyValueChanger(BaseCampaign):
    _definition = {'Daily_Probability': {'default': 1, 'description': 'The probability that an individual will move to the Target_Property_Value.', 'max': 1, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'NodePropertyValueChanger', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Maximum_Duration': {'default': 3.40282e+38, 'description': 'The maximum amount of time individuals have to move to a new group. This timing works in conjunction with Daily_Probability.', 'max': 3.40282e+38, 'min': -1, 'type': 'float'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Revert': {'default': 0, 'description': 'The number of days before an individual moves back to their original group.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'Sim_Types': ['*'], 'Target_NP_Key_Value': {'description': 'The NodeProperty key:value pair, as defined in the demographics file, to assign to the node.', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'class': 'NodePropertyValueChanger', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'NodePropertyValueChanger')

    def __init__(self, Daily_Probability=1, Disqualifying_Properties=[], Intervention_Name='NodePropertyValueChanger', Maximum_Duration=3.40282e+38, New_Property_Value=None, Revert=0, Sim_Types=['*'], Target_NP_Key_Value=None, iv_type='NodeTargeted', **kwargs):
        super(NodePropertyValueChanger, self).__init__(**kwargs)
        self.Daily_Probability = Daily_Probability
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Maximum_Duration = Maximum_Duration
        self.New_Property_Value = New_Property_Value
        self.Revert = Revert
        self.Sim_Types = Sim_Types
        self.Target_NP_Key_Value = Target_NP_Key_Value
        self.iv_type = iv_type



class Outbreak(BaseCampaign):
    _definition = {'Antigen': {'default': 0, 'description': 'The antigenic ID of the outbreak infection.', 'max': 10, 'min': 0, 'type': 'integer'}, 'Genome': {'default': 0, 'description': 'The genetic ID of the outbreak infection.', 'max': 16777200.0, 'min': -1, 'type': 'integer'}, 'Import_Age': {'default': 365, 'description': 'The age (in days) of infected import cases.', 'max': 43800, 'min': 0, 'type': 'float'}, 'Incubation_Period_Override': {'default': -1, 'description': 'The incubation period, in days, that infected individuals will go through before becoming infectious. This value overrides the incubation period set in the configuration file. Set to -1 to honor the configuration parameter settings.', 'max': 2147480000.0, 'min': -1, 'type': 'integer'}, 'Number_Cases_Per_Node': {'default': 1, 'description': 'The number of new cases of Outbreak to import (per node). ', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'Sim_Types': ['GENERIC_SIM', 'VECTOR_SIM', 'MALARIA_SIM', 'AIRBORNE_SIM', 'POLIO_SIM', 'TBHIV_SIM', 'STI_SIM', 'HIV_SIM', 'PY_SIM', 'TYPHOID_SIM'], 'class': 'Outbreak', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'Outbreak')

    def __init__(self, Antigen=0, Genome=0, Import_Age=365, Incubation_Period_Override=-1, Number_Cases_Per_Node=1, Sim_Types=['GENERIC_SIM', 'VECTOR_SIM', 'MALARIA_SIM', 'AIRBORNE_SIM', 'POLIO_SIM', 'TBHIV_SIM', 'STI_SIM', 'HIV_SIM', 'PY_SIM', 'TYPHOID_SIM'], iv_type='NodeTargeted', **kwargs):
        super(Outbreak, self).__init__(**kwargs)
        self.Antigen = Antigen
        self.Genome = Genome
        self.Import_Age = Import_Age
        self.Incubation_Period_Override = Incubation_Period_Override
        self.Number_Cases_Per_Node = Number_Cases_Per_Node
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class OutdoorRestKill(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'OutdoorRestKill', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration for the effects of killing of the targeted stage. Use a waning effect class to specify how this effect decays over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'OutdoorRestKill', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'OutdoorRestKill')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Intervention_Name='OutdoorRestKill', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(OutdoorRestKill, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class OvipositionTrap(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Habitat_Target': {'default': 'TEMPORARY_RAINFALL', 'description': 'The oviposition habitat type targeted by oviposition traps.', 'enum': ['TEMPORARY_RAINFALL', 'WATER_VEGETATION', 'HUMAN_POPULATION', 'CONSTANT', 'BRACKISH_SWAMP', 'MARSHY_STREAM', 'LINEAR_SPLINE', 'ALL_HABITATS'], 'type': 'enum'}, 'Intervention_Name': {'default': 'OvipositionTrap', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': "The configuration of the killing effects for the fraction of oviposition cycles that end in the female mosquito's death.  If there is skip oviposition, this is not the mortality per skip but the effective net mortality per gonotrophic cycle over all skips.", 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'OvipositionTrap', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'OvipositionTrap')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Habitat_Target=OvipositionTrap_Habitat_Target_Enum.TEMPORARY_RAINFALL, Intervention_Name='OvipositionTrap', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(OvipositionTrap, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Habitat_Target = Habitat_Target.name if isinstance(Habitat_Target, Enum) else Habitat_Target
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class ScaleLarvalHabitat(BaseCampaign):
    _definition = {'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'ScaleLarvalHabitat', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Larval_Habitat_Multiplier': {'description': 'The value by which to scale the larval habitat availability (specified in Larval_Habitat_Types in the configuration file) per intervention, across all habitat types, for specific habitat types, or for specific mosquito species within each habitat type. ', 'type': 'list', 'item_type': {'type': 'object'}, 'default': []}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'ScaleLarvalHabitat', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'ScaleLarvalHabitat')

    def __init__(self, Disqualifying_Properties=[], Intervention_Name='ScaleLarvalHabitat', Larval_Habitat_Multiplier=[], New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(ScaleLarvalHabitat, self).__init__(**kwargs)
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Larval_Habitat_Multiplier = Larval_Habitat_Multiplier
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class SpaceSpraying(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Habitat_Target': {'default': 'TEMPORARY_RAINFALL', 'description': 'The larval habitat type targeted for habitat reduction.', 'enum': ['TEMPORARY_RAINFALL', 'WATER_VEGETATION', 'HUMAN_POPULATION', 'CONSTANT', 'BRACKISH_SWAMP', 'MARSHY_STREAM', 'LINEAR_SPLINE', 'ALL_HABITATS'], 'type': 'enum'}, 'Intervention_Name': {'default': 'SpaceSpraying', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration of killing efficacy and waning for space spaying.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Reduction_Config': {'description': 'The configuration of larval habitat reduction and waning for space spraying. ', 'type': 'idmType:WaningEffect'}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'Spray_Kill_Target': {'default': 'SpaceSpray_FemalesOnly', 'description': 'The gender kill-target of vector control interventions.', 'enum': ['SpaceSpray_FemalesOnly', 'SpaceSpray_MalesOnly', 'SpaceSpray_FemalesAndMales', 'SpaceSpray_Indoor'], 'type': 'enum'}, 'class': 'SpaceSpraying', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'SpaceSpraying')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Habitat_Target=SpaceSpraying_Habitat_Target_Enum.TEMPORARY_RAINFALL, Intervention_Name='SpaceSpraying', Killing_Config=None, New_Property_Value=None, Reduction_Config=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], Spray_Kill_Target=SpaceSpraying_Spray_Kill_Target_Enum.SpaceSpray_FemalesOnly, iv_type='NodeTargeted', **kwargs):
        super(SpaceSpraying, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Habitat_Target = Habitat_Target.name if isinstance(Habitat_Target, Enum) else Habitat_Target
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Reduction_Config = Reduction_Config
        self.Sim_Types = Sim_Types
        self.Spray_Kill_Target = Spray_Kill_Target.name if isinstance(Spray_Kill_Target, Enum) else Spray_Kill_Target
        self.iv_type = iv_type



class SpatialRepellent(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'SpatialRepellent', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Repellency_Config': {'description': 'The configuration of efficacy and waning for spatial repellent.', 'type': 'idmType:WaningEffect'}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'SpatialRepellent', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'SpatialRepellent')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Intervention_Name='SpatialRepellent', New_Property_Value=None, Repellency_Config=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(SpatialRepellent, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.New_Property_Value = New_Property_Value
        self.Repellency_Config = Repellency_Config
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class SugarTrap(BaseCampaign):
    _definition = {'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vector control (unamortized).', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Intervention_Name': {'default': 'SugarTrap', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Killing_Config': {'description': 'The configuration for the effects of killing of the targeted stage. Use a waning effect class to specify how this effect decays over time.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::NodeProperties.*.Property':'<demographics>::NodeProperties.*.Values'"}, 'Sim_Types': ['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], 'class': 'SugarTrap', 'iv_type': 'NodeTargeted'}
    _validator = ClassValidator(_definition, 'SugarTrap')

    def __init__(self, Cost_To_Consumer=10, Disqualifying_Properties=[], Intervention_Name='SugarTrap', Killing_Config=None, New_Property_Value=None, Sim_Types=['VECTOR_SIM', 'MALARIA_SIM', 'DENGUE_SIM'], iv_type='NodeTargeted', **kwargs):
        super(SugarTrap, self).__init__(**kwargs)
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Intervention_Name = Intervention_Name
        self.Killing_Config = Killing_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.iv_type = iv_type



class NodeSetAll(BaseCampaign):
    _definition = {'class': 'NodeSetAll'}
    _validator = ClassValidator(_definition, 'NodeSetAll')

    def __init__(self, **kwargs):
        super(NodeSetAll, self).__init__(**kwargs)



class NodeSetNodeList(BaseCampaign):
    _definition = {'Node_List': {'description': 'A comma-separated list of node IDs in which this event will occur.', 'type': 'list', 'item_type': {'description': 'Id of Node', 'min': 0, 'type': 'integer'}, 'default': []}, 'class': 'NodeSetNodeList'}
    _validator = ClassValidator(_definition, 'NodeSetNodeList')

    def __init__(self, Node_List=[], **kwargs):
        super(NodeSetNodeList, self).__init__(**kwargs)
        self.Node_List = Node_List



class NodeSetPolygon(BaseCampaign):
    _definition = {'Polygon_Format': {'default': 'SHAPE', 'description': 'The type of polygon to create.', 'enum': ['SHAPE'], 'type': 'enum'}, 'Vertices': {'default': 'UNINITIALIZED STRING', 'description': 'Comma-separated list of latitude/longitude points that bound a polygon.', 'type': 'string'}, 'class': 'NodeSetPolygon'}
    _validator = ClassValidator(_definition, 'NodeSetPolygon')

    def __init__(self, Polygon_Format=NodeSetPolygon_Polygon_Format_Enum.SHAPE, Vertices='UNINITIALIZED STRING', **kwargs):
        super(NodeSetPolygon, self).__init__(**kwargs)
        self.Polygon_Format = Polygon_Format.name if isinstance(Polygon_Format, Enum) else Polygon_Format
        self.Vertices = Vertices



class WaningEffectBox(BaseCampaign):
    _definition = {'Box_Duration': {'default': 100, 'description': 'Box duration of effect in days.', 'max': 100000, 'min': 0, 'type': 'float'}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectBox'}
    _validator = ClassValidator(_definition, 'WaningEffectBox')

    def __init__(self, Box_Duration=100, Initial_Effect=1, **kwargs):
        super(WaningEffectBox, self).__init__(**kwargs)
        self.Box_Duration = Box_Duration
        self.Initial_Effect = Initial_Effect



class WaningEffectBoxExponential(BaseCampaign):
    _definition = {'Box_Duration': {'default': 100, 'description': 'Box duration of effect in days.', 'max': 100000, 'min': 0, 'type': 'float'}, 'Decay_Time_Constant': {'default': 100, 'description': 'The exponential decay length, in days.', 'max': 100000, 'min': 0, 'type': 'float'}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectBoxExponential'}
    _validator = ClassValidator(_definition, 'WaningEffectBoxExponential')

    def __init__(self, Box_Duration=100, Decay_Time_Constant=100, Initial_Effect=1, **kwargs):
        super(WaningEffectBoxExponential, self).__init__(**kwargs)
        self.Box_Duration = Box_Duration
        self.Decay_Time_Constant = Decay_Time_Constant
        self.Initial_Effect = Initial_Effect



class WaningEffectCombo(BaseCampaign):
    _definition = {'Add_Effects': {'default': 0, 'description': 'The Add_Effects parameter allows you to combine multiple effects from the waning effect classes. If set to true (1), then the waning effect values from the different waning effect objects are added together. If set to false (0), the waning effect values are multiplied. The resulting waning effect value must be greater than 1.', 'type': 'bool'}, 'Effect_List': {'description': 'A list of nested JSON objects to indicate how the intervention efficacy wanes over time.', '<WaningEffect Value>': {'type_name': 'idmType:WaningEffect', 'type_schema': {'base': 'interventions.idmType.WaningEffect'}}}, 'Expires_When_All_Expire': {'default': 0, 'description': 'If set to true (1), then all of the effects, as specified in the Effect_List parameter, must expire for the efficacy of the intervention to expire. If set to false (0), then the efficacy of the intervention will expire as soon as one of the parameters expires.', 'type': 'bool'}, 'class': 'WaningEffectCombo'}
    _validator = ClassValidator(_definition, 'WaningEffectCombo')

    def __init__(self, Add_Effects=False, Effect_List=None, Expires_When_All_Expire=False, **kwargs):
        super(WaningEffectCombo, self).__init__(**kwargs)
        self.Add_Effects = Add_Effects
        self.Effect_List = Effect_List
        self.Expires_When_All_Expire = Expires_When_All_Expire



class WaningEffectConstant(BaseCampaign):
    _definition = {'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectConstant'}
    _validator = ClassValidator(_definition, 'WaningEffectConstant')

    def __init__(self, Initial_Effect=1, **kwargs):
        super(WaningEffectConstant, self).__init__(**kwargs)
        self.Initial_Effect = Initial_Effect


class MeaslesVaccine(BaseCampaign):
    _definition = {'Acquire_Config': {'description': 'The configuration for measles vaccine acquisition.', 'type': 'idmType:WaningEffect'}, 'Existing_Antibody_Blocking_Multiplier':{'default': 0.0, 'description': 'If a child has existing antibodies, maternal or otherwise, take rate of measles vaccine is multiplied by this number', 'max': 999999, 'min': 0, 'type': 'float'}, 'Cost_To_Consumer': {'default': 10, 'description': 'Unit cost per vaccine (unamortized). ', 'max': 999999, 'min': 0, 'type': 'float'}, 'Disqualifying_Properties': {'default': [], 'description': 'A list of IndividualProperty key:value pairs that cause an intervention to be aborted. Generally used to control the flow of health care access. For example, to prevent the same individual from accessing health care via two different routes at the same time.', 'type': 'Dynamic String Set', 'value_source': ''}, 'Dont_Allow_Duplicates': {'default': 0, 'description': "If an individual's container has an intervention, set to true (1) to prevent them from receiving another copy of the intervention. Supported by all intervention classes.", 'type': 'bool'}, 'Intervention_Name': {'default': 'MeaslesVaccine', 'description': 'The optional name used to refer to this intervention as a means to differentiate it from others that use the same class.', 'type': 'string'}, 'Mortality_Config': {'description': 'The configuration for multi-effect vaccine mortality.', 'type': 'idmType:WaningEffect'}, 'New_Property_Value': {'description': 'An optional IndividualProperty key:value pair that will be assigned when the intervention is distributed. Generally used to indicate the broad category of health care cascade to which an intervention belongs to prevent individuals from accessing care through multiple pathways. ', 'type': 'Constrained String', 'value_source': "'<demographics>::*.Individual_Properties.*.Property':'<demographics>::*.Individual_Properties.*.Values'"}, 'Sim_Types': ['*'], 'Transmit_Config': {'description': 'The configuration for multi-effect vaccine transmission.', 'type': 'idmType:WaningEffect'}, 'Vaccine_Take_Vs_Age_Map': {'default': {'Times': [0, 99999], 'Values':[0, 1]}, 'description': 'Age-dependent rate at which delivered vaccines will successfully stimulate an immune response and achieve the desired efficacy.  Times represents age at receipt, Values represents take rate at that age.  Take is linearly interpolated between age points', 'type': 'dict'}, 'class': 'MultiEffectVaccine', 'iv_type': 'IndividualTargeted'}
    _validator = ClassValidator(_definition, 'MeaslesVaccine')

    def __init__(self, Acquire_Config=None, Cost_To_Consumer=10, Disqualifying_Properties=[], Dont_Allow_Duplicates=False, Existing_Antibody_Blocking_Multiplier =None, Intervention_Name='MultiEffectVaccine', Mortality_Config=None, New_Property_Value=None, Sim_Types=['*'], Transmit_Config=None, Vaccine_Take_Vs_Age_Map = None, iv_type='IndividualTargeted', **kwargs):
        super(MeaslesVaccine, self).__init__(**kwargs)
        self.Acquire_Config = WaningEffectConstant(Initial_Effect=1.0) if Acquire_Config is None else Acquire_Config
        self.Cost_To_Consumer = Cost_To_Consumer
        self.Disqualifying_Properties = Disqualifying_Properties
        self.Dont_Allow_Duplicates = Dont_Allow_Duplicates
        self.Existing_Antibody_Blocking_Multiplier = 0.4 if Existing_Antibody_Blocking_Multiplier is None else Existing_Antibody_Blocking_Multiplier
        self.Intervention_Name = Intervention_Name
        self.Mortality_Config = Mortality_Config
        self.New_Property_Value = New_Property_Value
        self.Sim_Types = Sim_Types
        self.Transmit_Config = Transmit_Config
        if Vaccine_Take_Vs_Age_Map is None:
            self.Vaccine_Take_Vs_Age_Map = {'Times': [0, 90, 120, 150, 180, 210, 240, 270, 300, 330, 365, 100000],
                                            'Values': [0, 0, 0.53, 0.66, 0.77, 0.82, 0.89, 0.95, 0.95, 0.95, 0.95, 0.95]}
        else:
            self.Vaccine_Take_Vs_Age_Map = Vaccine_Take_Vs_Age_Map
        self.iv_type = iv_type


class WaningEffectExponential(BaseCampaign):
    _definition = {'Decay_Time_Constant': {'default': 100, 'description': 'The exponential decay length, in days.', 'max': 100000, 'min': 0, 'type': 'float'}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectExponential'}
    _validator = ClassValidator(_definition, 'WaningEffectExponential')

    def __init__(self, Decay_Time_Constant=100, Initial_Effect=1, **kwargs):
        super(WaningEffectExponential, self).__init__(**kwargs)
        self.Decay_Time_Constant = Decay_Time_Constant
        self.Initial_Effect = Initial_Effect



class WaningEffectMapCount(BaseCampaign):
    _definition = {'Durability_Map': {'description': 'The time, in days, since the intervention was distributed and a multiplier for the Initial_Effect.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectMapCount'}
    _validator = ClassValidator(_definition, 'WaningEffectMapCount')

    def __init__(self, Durability_Map=None, Initial_Effect=1, **kwargs):
        super(WaningEffectMapCount, self).__init__(**kwargs)
        self.Durability_Map = Durability_Map
        self.Initial_Effect = Initial_Effect



class WaningEffectMapLinear(BaseCampaign):
    _definition = {'Durability_Map': {'description': 'The time, in days, since the intervention was distributed and a multiplier for the Initial_Effect.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Expire_At_Durability_Map_End': {'default': 0, 'description': 'Set to 1 to let the intervention expire when the end of the map is reached.', 'type': 'bool'}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'Reference_Timer': {'default': 0, 'description': 'Timestamp at which linear-map should be anchored.', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'class': 'WaningEffectMapLinear'}
    _validator = ClassValidator(_definition, 'WaningEffectMapLinear')

    def __init__(self, Durability_Map=None, Expire_At_Durability_Map_End=False, Initial_Effect=1, Reference_Timer=0, **kwargs):
        super(WaningEffectMapLinear, self).__init__(**kwargs)
        self.Durability_Map = Durability_Map
        self.Expire_At_Durability_Map_End = Expire_At_Durability_Map_End
        self.Initial_Effect = Initial_Effect
        self.Reference_Timer = Reference_Timer



class WaningEffectMapLinearAge(BaseCampaign):
    _definition = {'Durability_Map': {'description': 'The time, in days, since the intervention was distributed and a multiplier for the Initial_Effect.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectMapLinearAge'}
    _validator = ClassValidator(_definition, 'WaningEffectMapLinearAge')

    def __init__(self, Durability_Map=None, Initial_Effect=1, **kwargs):
        super(WaningEffectMapLinearAge, self).__init__(**kwargs)
        self.Durability_Map = Durability_Map
        self.Initial_Effect = Initial_Effect



class WaningEffectMapLinearSeasonal(BaseCampaign):
    _definition = {'Durability_Map': {'description': 'The time, in days, since the intervention was distributed and a multiplier for the Initial_Effect.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectMapLinearSeasonal'}
    _validator = ClassValidator(_definition, 'WaningEffectMapLinearSeasonal')

    def __init__(self, Durability_Map=None, Initial_Effect=1, **kwargs):
        super(WaningEffectMapLinearSeasonal, self).__init__(**kwargs)
        self.Durability_Map = Durability_Map
        self.Initial_Effect = Initial_Effect



class WaningEffectMapPiecewise(BaseCampaign):
    _definition = {'Durability_Map': {'description': 'The time, in days, since the intervention was distributed and a multiplier for the Initial_Effect.', 'Times': {'type': 'list', 'item_type': {'description': 'An array of years.', 'max': 999999, 'min': 0, 'type': 'float'}, 'default': []}, 'Values': {'type': 'list', 'item_type': {'description': 'An array of values to match the defined Times.', 'max': 3.40282e+38, 'min': 0, 'type': 'float'}, 'default': []}}, 'Expire_At_Durability_Map_End': {'default': 0, 'description': 'Set to 1 to let the intervention expire when the end of the map is reached.', 'type': 'bool'}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'Reference_Timer': {'default': 0, 'description': 'Timestamp at which linear-map should be anchored.', 'max': 2147480000.0, 'min': 0, 'type': 'integer'}, 'class': 'WaningEffectMapPiecewise'}
    _validator = ClassValidator(_definition, 'WaningEffectMapPiecewise')

    def __init__(self, Durability_Map=None, Expire_At_Durability_Map_End=False, Initial_Effect=1, Reference_Timer=0, **kwargs):
        super(WaningEffectMapPiecewise, self).__init__(**kwargs)
        self.Durability_Map = Durability_Map
        self.Expire_At_Durability_Map_End = Expire_At_Durability_Map_End
        self.Initial_Effect = Initial_Effect
        self.Reference_Timer = Reference_Timer



class WaningEffectRandomBox(BaseCampaign):
    _definition = {'Expected_Discard_Time': {'default': 100, 'description': 'The mean time, in days, of an exponential distribution of the duration of the effect of an intervention (such as a vaccine or bed net).', 'max': 100000, 'min': 0, 'type': 'float'}, 'Initial_Effect': {'default': 1, 'description': 'Initial strength of the effect. The effect decays over time.', 'max': 1, 'min': 0, 'type': 'float'}, 'class': 'WaningEffectRandomBox'}
    _validator = ClassValidator(_definition, 'WaningEffectRandomBox')

    def __init__(self, Expected_Discard_Time=100, Initial_Effect=1, **kwargs):
        super(WaningEffectRandomBox, self).__init__(**kwargs)
        self.Expected_Discard_Time = Expected_Discard_Time
        self.Initial_Effect = Initial_Effect

###################################
# Our classes
###################################


class Campaign(BaseCampaign):
    _definition = {"Campaign_Name": "Empty Campaign", "Events": [], "Use_Defaults": {"default": True, "type": "bool"}}
    _validator = ClassValidator(_definition, 'Campaign')

    def __init__(self, Campaign_Name="Empty Campaign", Use_Defaults=True, Events=[], **kwargs):
        super(Campaign, self).__init__(**kwargs)
        self.Campaign_Name = Campaign_Name
        self.Use_Defaults = Use_Defaults
        self.Events = Events

    def add_campaign_event(self, ce):
        self.Events.append(ce)